﻿' Autor: Frank Schüler
' Datum: 01.01.2018
' Status: Komplett 2

Option Strict On
Option Explicit On

Imports System.Runtime.InteropServices

Public Class EnumPortableDeviceObjectIDs
    Implements IDisposable

    Private Const IID_IEnumPortableDeviceObjectIDs As String = "10ECE955-CF41-4728-BFA0-41EEDF1BBF19"
    Private Const CLSID_EnumPortableDeviceObjectIDs As String = ""
    Private Const S_OK As Integer = 0

    Public _ifc As IEnumPortableDeviceObjectIDs

#Region "Interface IEnumPortableDeviceObjectIDs"
    ' https://msdn.microsoft.com/en-us/library/windows/desktop/dd319350(v=vs.85).aspx
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IEnumPortableDeviceObjectIDs)>
    Public Interface IEnumPortableDeviceObjectIDs
        Function [Next](<[In]> ByVal cObjects As UInteger,
                    <[In], Out, MarshalAs(UnmanagedType.LPArray, ArraySubType:=UnmanagedType.LPWStr, SizeParamIndex:=1)> ByVal pObjIDs As String(),
                    <[In], Out> ByRef pcFetched As UInteger) As Integer

        Function Skip(<[In]> ByVal cObjects As UInteger) As Integer

        Function Reset() As Integer

        Function Clone(<Out, MarshalAs(UnmanagedType.Interface)> ByRef ppEnum As EnumPortableDeviceObjectIDs.IEnumPortableDeviceObjectIDs) As Integer

        Function Cancel() As Integer

    End Interface
#End Region

#Region "Interface Helper"
    ' Retrieves the next one or more object IDs in the enumeration sequence.
    Function [Next](ByVal cObjects As UInteger,
                    ByRef pObjIDs As String(),
                    ByRef pcFetched As UInteger) As Boolean

        Array.Resize(pObjIDs, Convert.ToInt32(cObjects))
        Dim ret = _ifc.Next(cObjects, pObjIDs, pcFetched)
        If ret = S_OK Then
            Return True
        ElseIf ret <> S_OK Then
            Throw New COMException("Next faild", ret)
        End If
        Return False
    End Function

    ' Skips a specified number of objects in the enumeration sequence.
    Function Skip(ByVal cObjects As UInteger) As Boolean
        Dim ret = _ifc.Skip(cObjects)
        If ret = S_OK Then
            Return True
        ElseIf ret < S_OK Then
            Throw New COMException("Skip faild", ret)
        End If
        Return False
    End Function

    ' Resets the enumeration sequence to the beginning.
    Function Reset() As Boolean
        Dim ret = _ifc.Reset
        If ret = S_OK Then
            Return True
        ElseIf ret < S_OK Then
            Throw New COMException("Reset faild", ret)
        End If
        Return False
    End Function

    ' Duplicates the current IEnumPortableDeviceObjectIDs interface.
    ' Not implemented in this release.
    Function Clone() As EnumPortableDeviceObjectIDs
        Dim ppEnum As New EnumPortableDeviceObjectIDs
        Dim ret = _ifc.Clone(ppEnum._ifc)
        If ret = S_OK Then
            Return ppEnum
        ElseIf ret < S_OK Then
            Throw New COMException("Clone faild", ret)
        End If
        Return Nothing
    End Function

    ' Cancels a pending operation on the interface.
    Function Cancel() As Boolean
        Dim ret = _ifc.Cancel
        If ret = S_OK Then
            Return True
        ElseIf ret < S_OK Then
            Throw New COMException("Cancel faild", ret)
        End If
        Return False
    End Function
#End Region

#Region "IDisposable Support"
    Private disposedValue As Boolean

    Protected Overridable Sub Dispose(disposing As Boolean)
        If Not disposedValue Then
            If disposing Then
                ' TODO: verwalteten Zustand (verwaltete Objekte) entsorgen.
            End If

            If _ifc IsNot Nothing Then
                Marshal.ReleaseComObject(_ifc)
                _ifc = Nothing
            End If
        End If
        disposedValue = True
    End Sub

    Protected Overrides Sub Finalize()
        Dispose(False)
        MyBase.Finalize()
    End Sub

    Public Sub Dispose() Implements IDisposable.Dispose
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub
#End Region

End Class
