﻿' Autor: Frank Schüler
' Datum: 01.01.2018

Option Strict On
Option Explicit On

Imports System.IO
Imports System.ComponentModel

Public Class Form1

    Private sDevID As String = String.Empty
    Private sPnpDevIDs As String() = New String() {}
    Private boolConnected As Boolean = False
    Private myMenu As ContextMenuStrip
    Private currIPDev As PortableDevice
    Private currIPDevValues As PortableDeviceValues
    Private currIPDevContent As PortableDeviceContent

    Private Sub Form1_Load(sender As Object, e As EventArgs) Handles Me.Load
        gbContent.Controls.Add(tvDir)
        gbContent.Controls.Add(lbFile)
        RefreshDeviceList()
    End Sub

    Private Sub Form1_Closing(sender As Object, e As CancelEventArgs) Handles Me.Closing
        If myMenu IsNot Nothing Then
            myMenu.Dispose()
        End If
        If boolConnected = True Then
            If currIPDev.Cancel = True Then
                If currIPDev.Close = True Then
                    currIPDevContent.Dispose()
                    currIPDevValues.Dispose()
                    currIPDev.Dispose()
                End If
            End If
        End If
    End Sub

    Private Sub Refresh_Click(sender As Object, e As EventArgs) Handles buttRefresh.Click
        RefreshDeviceList()
    End Sub

    Private Sub cbStorage_SelectedIndexChanged(sender As Object, e As EventArgs) Handles cbStorage.SelectedIndexChanged
        Dim sarrSelectedStorage As String() = cbStorage.Items.Item(cbStorage.SelectedIndex).ToString.Split(CChar("("))
        Dim sSelectedStorage As String = sarrSelectedStorage(0).Trim
        If sSelectedStorage <> String.Empty Then
            GetSupportedProperties(currIPDevContent, sSelectedStorage, lbStorage)
        End If
    End Sub

    Private Sub cbDrive_SelectedIndexChanged(sender As Object, e As EventArgs) Handles cbDrive.SelectedIndexChanged
        Dim sarrSelectedDrive As String() = cbDrive.Items.Item(cbDrive.SelectedIndex).ToString.Split(CChar("("))
        Dim sObjID As String = sarrSelectedDrive(0).Trim
        Dim sObjName As String = sarrSelectedDrive(1).Substring(0, sarrSelectedDrive(1).Length - 1)
        If sObjID <> String.Empty AndAlso sObjName <> String.Empty Then
            GetContentFolder(currIPDevContent, sObjID, sObjName)
            GetContentFiles(currIPDevContent, sObjID)
        End If
    End Sub

    Private Sub Connect_Click(sender As Object, e As EventArgs) Handles buttConnect.Click
        If boolConnected = False Then
            sDevID = sPnpDevIDs(cbWpdDevice.SelectedIndex)
            currIPDevValues = New PortableDeviceValues
            currIPDevValues.SetStringValue(GetPropertyKey(PropertyKeys.WPD_CLIENT_NAME), "WPD-Demo")
            currIPDevValues.SetStringValue(GetPropertyKey(PropertyKeys.WPD_CLIENT_MAJOR_VERSION), "1")
            currIPDevValues.SetStringValue(GetPropertyKey(PropertyKeys.WPD_CLIENT_MINOR_VERSION), "0")
            currIPDevValues.SetStringValue(GetPropertyKey(PropertyKeys.WPD_CLIENT_REVISION), "0")
            currIPDev = New PortableDevice
            If currIPDev.Open(sDevID, currIPDevValues) = True Then
                boolConnected = True
                buttConnect.Text = "Disconnect"
                buttRefresh.Enabled = False
                cbWpdDevice.Enabled = False
                Me.Text = "VB.NET WPD-Demo: Connected to " & cbWpdDevice.Text
                currIPDevContent = currIPDev.Content
                GetSupportedProperties(currIPDevContent, WPD_DEVICE_OBJECT_ID, lbProperties)
                GetSupportedCommands(currIPDev)
                ResetDevice(currIPDev)
                ContentLocation(currIPDev)
                StorageEject(currIPDev)
                StorageFormat(currIPDev)
                SmsSend(currIPDev)
                StillImageCapture(currIPDev)
                ListDeviceStorage(currIPDevContent)
            End If
        Else
            If currIPDev.Cancel = True Then
                If currIPDev.Close = True Then
                    sDevID = String.Empty
                    boolConnected = False
                    buttConnect.Text = "Connect"
                    buttRefresh.Enabled = True
                    cbWpdDevice.Enabled = True
                    Me.Text = "VB.NET WPD-Demo:"
                    For Each gBox In Me.Controls.OfType(Of GroupBox)()
                        For Each chBox In gBox.Controls.OfType(Of CheckBox)()
                            chBox.Checked = False
                        Next
                        For Each coBox In gBox.Controls.OfType(Of ComboBox)()
                            coBox.Text = String.Empty
                            coBox.Items.Clear()
                        Next
                        For Each liBox In gBox.Controls.OfType(Of ListBox)()
                            liBox.Items.Clear()
                        Next
                        For Each trView In gBox.Controls.OfType(Of TreeView)()
                            trView.Nodes.Clear()
                        Next
                    Next
                    currIPDevContent.Dispose()
                    currIPDevValues.Dispose()
                    currIPDev.Dispose()
                End If
            End If
        End If
    End Sub

    Public Sub ResetContextMenu()
        If myMenu IsNot Nothing Then
            myMenu.Dispose()
        End If
        myMenu = New ContextMenuStrip
    End Sub

    Public Sub CreateContextMenu(ByVal sObjID As String)
        Dim mItems As ToolStripMenuItem() = New ToolStripMenuItem() {}
        Select Case GetPropertyStr(currIPDevContent, sObjID, GetPropertyKey(PropertyKeys.WPD_OBJECT_CONTENT_TYPE))
            Case WPD_CONTENT_TYPE_IMAGE
                Array.Resize(mItems, 4)
                mItems(0) = CreateMenuItem("View Supported Properties", "viewsupp/" & sObjID)
                mItems(1) = CreateMenuItem("Rename File", "rename/" & sObjID)
                mItems(2) = CreateMenuItem("Copy File from Device", "copyfrom/" & sObjID)
                mItems(3) = CreateMenuItem("View Image", "viewimg/" & sObjID)

            Case WPD_CONTENT_TYPE_AUDIO
                Array.Resize(mItems, 3)
                mItems(0) = CreateMenuItem("View Supported Properties", "viewsupp/" & sObjID)
                mItems(1) = CreateMenuItem("Rename File", "rename/" & sObjID)
                mItems(2) = CreateMenuItem("Copy File from Device", "copyfrom/" & sObjID)

            Case WPD_CONTENT_TYPE_VIDEO
                Array.Resize(mItems, 3)
                mItems(0) = CreateMenuItem("View Supported Properties", "viewsupp/" & sObjID)
                mItems(1) = CreateMenuItem("Rename File", "rename/" & sObjID)
                mItems(2) = CreateMenuItem("Copy File from Device", "copyfrom/" & sObjID)

            Case WPD_CONTENT_TYPE_UNSPECIFIED
                Array.Resize(mItems, 4)
                mItems(0) = CreateMenuItem("View Supported Properties", "viewsupp/" & sObjID)
                mItems(1) = CreateMenuItem("Rename File", "rename/" & sObjID)
                mItems(2) = CreateMenuItem("Copy File from Device", "copyfrom/" & sObjID)
                mItems(3) = CreateMenuItem("Delete File from Device", "deletefrom/" & sObjID)

            Case Else
                Array.Resize(mItems, 1)
                mItems(0) = CreateMenuItem("View Supported Properties", "viewsupp/" & sObjID)
        End Select
        For iItem = 0 To mItems.Count - 2
            myMenu.Items.Add(mItems(iItem))
            myMenu.Items.Add(New ToolStripSeparator)
        Next
        myMenu.Items.Add(mItems(mItems.Count - 1))
    End Sub

    Private Function CreateMenuItem(ByVal text As String,
            Optional ByVal name As String = "") As ToolStripMenuItem

        ' Einen Event Handler erstellen
        Dim evH As New EventHandler(AddressOf ContextMenu_Click)
        ' Ein neues MenuItem erzeugen
        Dim tsmi As New ToolStripMenuItem(text, Nothing, evH, name)

        Return tsmi
    End Function

    Private Sub ContextMenu_Click(ByVal sender As Object, ByVal e As EventArgs)
        Dim tsmi As ToolStripMenuItem = DirectCast(sender, ToolStripMenuItem)
        Dim sArgs As String() = tsmi.Name.Split(CChar("/"))
        Dim sCommand As String = sArgs(0)
        Dim sObjID As String = sArgs(1)
        Select Case sCommand
            Case "rename"
                mnuRename(currIPDevContent, sObjID)
            Case "copyfrom"
                mnuCopyFrom(currIPDevContent, sObjID)
            Case "deletefrom"
                mnuDeleteFrom(currIPDevContent, sObjID)
            Case "viewsupp"
                mnuViewSupp(currIPDevContent, sObjID)
            Case "viewimg"
                mnuViewImg(currIPDevContent, sObjID)
            Case Else
                ' do nothing
        End Select
    End Sub

    Private Sub mnuDeleteFrom(ByVal Content As PortableDeviceContent, ByVal ObjID As String)

        Dim sParentID As String = GetPropertyStr(Content, ObjID, GetPropertyKey(PropertyKeys.WPD_OBJECT_PARENT_ID))

        If GetPropertyBool(Content, ObjID, GetPropertyKey(PropertyKeys.WPD_OBJECT_CAN_DELETE)) = True Then

            Using ObjectsToDelete As PortableDevicePropVariantCollection = New PortableDevicePropVariantCollection
                Dim pv As WpdStruct.PROPVARIANT
                pv.vt = WpdStruct.VARENUM.VT_LPWSTR
                pv.pwszVal = ObjID
                If ObjectsToDelete.Add(pv) = True Then
                    Using ObjectsFailToDelete As PortableDevicePropVariantCollection = Content.Delete(WpdStruct.DELETE_OBJECT_OPTIONS.PORTABLE_DEVICE_DELETE_NO_RECURSION, ObjectsToDelete)
                        GetContentFiles(Content, sParentID)
                    End Using
                End If
            End Using
        End If
    End Sub

    Private Sub mnuRename(ByVal Content As PortableDeviceContent, ByVal ObjId As String)
        Dim bOK As Boolean = False
        Dim sNewFileName As String
        Dim sNewName As String
        Dim sOrgFileName As String = GetPropertyStr(Content, ObjId, GetPropertyKey(PropertyKeys.WPD_OBJECT_ORIGINAL_FILE_NAME))
        Dim sParentID As String = GetPropertyStr(Content, ObjId, GetPropertyKey(PropertyKeys.WPD_OBJECT_PARENT_ID))
        Using frm As New frmRename
            frm.tbFilename.Text = Path.GetFileNameWithoutExtension(sOrgFileName)
            frm.tbFileext.Text = Path.GetExtension(sOrgFileName).Substring(1)
            frm.ShowDialog()
            sNewName = frm.tbFilename.Text.Trim
            sNewFileName = frm.tbFilename.Text.Trim & "." & frm.tbFileext.Text.Trim
            bOK = frm.bOK
        End Using
        If bOK = True AndAlso sNewFileName <> sOrgFileName Then
            Using IPDevProperties As PortableDeviceProperties = Content.Properties
                Using IPDevValuesOrg As PortableDeviceValues = IPDevProperties.GetPropertyAttributes(ObjId, GetPropertyKey(PropertyKeys.WPD_OBJECT_ORIGINAL_FILE_NAME))
                    If IPDevValuesOrg.GetBoolValue(GetPropertyKey(PropertyKeys.WPD_PROPERTY_ATTRIBUTE_CAN_WRITE)) = True Then
                        Using IPDevValuesNew As New PortableDeviceValues
                            If IPDevValuesNew.SetStringValue(GetPropertyKey(PropertyKeys.WPD_OBJECT_ORIGINAL_FILE_NAME), sNewFileName) = True Then
                                If IPDevValuesNew.SetStringValue(GetPropertyKey(PropertyKeys.WPD_OBJECT_NAME), sNewName) = True Then
                                    Using PropWriteResults As PortableDeviceValues = IPDevProperties.SetValues(ObjId, IPDevValuesNew)
                                        If sNewFileName <> sOrgFileName Then
                                            GetContentFiles(Content, sParentID)
                                            MsgBox("File '" & sOrgFileName & "' renamed to '" & sNewFileName & "'.", MsgBoxStyle.Information Or MsgBoxStyle.OkOnly, "Rename File")
                                        End If
                                    End Using
                                End If
                            End If
                        End Using
                    End If
                End Using
            End Using
        End If
    End Sub

    Private Sub mnuCopyFrom(ByVal Content As PortableDeviceContent, ByVal ObjId As String)
        Using IPDevResources As PortableDeviceResources = Content.Transfer
            Dim optimalTransferSize As UInteger
            Using IContentStream As Stream = IPDevResources.GetStream(ObjId, GetPropertyKey(PropertyKeys.WPD_RESOURCE_DEFAULT),
                                                                       WpdStruct.STGM.STGM_READ, optimalTransferSize)
                Dim fileSize = GetPropertyInt(Content, ObjId, GetPropertyKey(PropertyKeys.WPD_OBJECT_SIZE))
                Dim fileName = GetPropertyStr(Content, ObjId, GetPropertyKey(PropertyKeys.WPD_OBJECT_ORIGINAL_FILE_NAME))
                Using fsd As New SaveFileDialog
                    fsd.Title = "Save File"
                    fsd.FileName = fileName
                    fsd.ShowDialog()
                    If fsd.FileName <> "" Then
                        If fileSize > 0 Then
                            Dim copyfileBuffer As Boolean = False
                            Dim fileBuffer As Byte() = New Byte(fileSize - 1) {}
                            If fileSize <= optimalTransferSize Then
                                If IContentStream.Read(fileBuffer, CUInt(fileSize)) = fileSize Then
                                    copyfileBuffer = True
                                End If
                            Else
                                Dim copyItem As UInteger
                                Dim copyCount As UInteger = CUInt(fileSize \ optimalTransferSize)
                                Dim lastBytes As UInteger = CUInt(fileSize - (copyCount * optimalTransferSize))
                                Dim tmpfileBuffer As Byte() = New Byte(CInt(optimalTransferSize - 1)) {}
                                For copyItem = 0 To CUInt(copyCount - 1)
                                    If IContentStream.Read(tmpfileBuffer, optimalTransferSize) = optimalTransferSize Then
                                        copyfileBuffer = True
                                        Array.Copy(tmpfileBuffer, 0, fileBuffer, optimalTransferSize * copyItem, optimalTransferSize)
                                    Else
                                        copyfileBuffer = False
                                    End If
                                Next copyItem
                                Array.Resize(tmpfileBuffer, CInt(lastBytes))
                                If IContentStream.Read(tmpfileBuffer, lastBytes) = lastBytes Then
                                    copyfileBuffer = True
                                    Array.Copy(tmpfileBuffer, 0, fileBuffer, optimalTransferSize * copyItem, lastBytes)
                                Else
                                    copyfileBuffer = False
                                End If
                                tmpfileBuffer = Nothing
                            End If
                            If copyfileBuffer = True Then
                                Using fs As FileStream = CType(fsd.OpenFile(), System.IO.FileStream)
                                    fs.Write(fileBuffer, 0, fileBuffer.Length)
                                    fs.Close()
                                End Using
                                fileBuffer = Nothing
                                MsgBox("File under '" & fsd.FileName & "' saved.", MsgBoxStyle.Information Or MsgBoxStyle.OkOnly, "Save File")
                            End If
                        End If
                    End If
                End Using
            End Using
        End Using
    End Sub

    Private Sub mnuViewImg(ByVal Content As PortableDeviceContent, ByVal ObjId As String)
        Using IPDevResources As PortableDeviceResources = Content.Transfer
            Dim optimalTransferSize As UInteger
            Using IContentStream As Stream = IPDevResources.GetStream(ObjId, GetPropertyKey(PropertyKeys.WPD_RESOURCE_DEFAULT),
                                                                       WpdStruct.STGM.STGM_READ, optimalTransferSize)
                Dim fileSize = GetPropertyInt(Content, ObjId, GetPropertyKey(PropertyKeys.WPD_OBJECT_SIZE))
                If fileSize > 0 Then
                    Dim copyImgBuffer As Boolean = False
                    Dim imgBuffer As Byte() = New Byte(fileSize - 1) {}
                    If fileSize <= optimalTransferSize Then
                        If IContentStream.Read(imgBuffer, CUInt(fileSize)) = fileSize Then
                            copyImgBuffer = True
                        End If
                    Else
                        Dim copyItem As UInteger
                        Dim copyCount As UInteger = CUInt(fileSize \ optimalTransferSize)
                        Dim lastBytes As UInteger = CUInt(fileSize - (copyCount * optimalTransferSize))
                        Dim tmpImgBuffer As Byte() = New Byte(CInt(optimalTransferSize - 1)) {}
                        For copyItem = 0 To CUInt(copyCount - 1)
                            If IContentStream.Read(tmpImgBuffer, optimalTransferSize) = optimalTransferSize Then
                                copyImgBuffer = True
                                Array.Copy(tmpImgBuffer, 0, imgBuffer, optimalTransferSize * copyItem, optimalTransferSize)
                            Else
                                copyImgBuffer = False
                            End If
                        Next copyItem
                        Array.Resize(tmpImgBuffer, CInt(lastBytes))
                        If IContentStream.Read(tmpImgBuffer, lastBytes) = lastBytes Then
                            copyImgBuffer = True
                            Array.Copy(tmpImgBuffer, 0, imgBuffer, optimalTransferSize * copyItem, lastBytes)
                        Else
                            copyImgBuffer = False
                        End If
                        tmpImgBuffer = Nothing
                    End If
                    If copyImgBuffer = True Then
                        Dim memStream As New IO.MemoryStream(imgBuffer)
                        Dim newImg As Image = Image.FromStream(memStream)
                        Using frm As New frmImage
                            Using outImg As Image = New Bitmap(frm.picImage.Width, frm.picImage.Height, 0,
                                                               Imaging.PixelFormat.Format32bppArgb, IntPtr.Zero)
                                Using gra As Graphics = Graphics.FromImage(outImg)
                                    gra.DrawImage(newImg, 0, 0, frm.picImage.Width, frm.picImage.Height)
                                End Using
                                newImg.Dispose()
                                memStream.Dispose()
                                imgBuffer = Nothing
                                frm.picImage.Image = outImg
                                frm.ShowDialog(Me)
                            End Using
                        End Using
                    End If
                End If
            End Using
        End Using
    End Sub

    Private Sub mnuViewSupp(ByVal Content As PortableDeviceContent, ByVal sObjID As String)
        Using frm As New frmProperties
            frm.Text = "Supported Properties ObjectID: " & sObjID
            GetSupportedProperties(Content, sObjID, frm.lbProperties)
            frm.ShowDialog(Me)
        End Using
    End Sub

    Private Sub tvDir_AfterExpand(sender As Object, e As TreeViewEventArgs) Handles tvDir.AfterExpand
        tvDir.SelectedNode = e.Node
        EnumerateContent(currIPDevContent, e.Node.Name, 1, True, True)
        GetContentFiles(currIPDevContent, e.Node.Name)
    End Sub

    Private Sub tvDir_NodeMouseClick(sender As Object, e As TreeNodeMouseClickEventArgs) Handles tvDir.NodeMouseClick
        If e.Node.IsSelected = False Then
            tvDir.SelectedNode = e.Node
            lbFile.Items.Clear()
            GetContentFiles(currIPDevContent, e.Node.Name)
        End If
    End Sub

    Private Sub tvDir_MouseUp(sender As Object, e As MouseEventArgs) Handles tvDir.MouseUp
        If e.Button = MouseButtons.Right Then
            Dim xNode = tvDir.HitTest(New Point(e.X, e.Y)).Node
            If xNode IsNot Nothing Then
                Dim sMnu_ObjID As String = xNode.Name
                ResetContextMenu()
                CreateContextMenu(sMnu_ObjID)
                myMenu.Show(tvDir, e.Location)
            End If
        End If
    End Sub

    Private Sub lbFile_MouseUp(sender As Object, e As MouseEventArgs) Handles lbFile.MouseUp
        If lbFile.Items.Count > 0 Then
            If e.Button = MouseButtons.Right Then
                Dim sarrSelectedItem As String() = lbFile.SelectedItem.ToString.Split(CChar("("))
                Dim sMnu_ObjID As String = sarrSelectedItem(1).Substring(0, sarrSelectedItem(1).Length - 1)
                ResetContextMenu()
                CreateContextMenu(sMnu_ObjID)
                myMenu.Show(lbFile, e.Location)
            End If
        End If
    End Sub

    Private Sub lbFile_MouseDown(sender As Object, e As MouseEventArgs) Handles lbFile.MouseDown
        If lbFile.Items.Count > 0 Then
            If e.Button = MouseButtons.Right Then
                lbFile.SelectedIndex = lbFile.IndexFromPoint(New Point(e.X, e.Y))
            End If
        End If
    End Sub

    Private Sub RefreshDeviceList()
        Array.Clear(sPnpDevIDs, 0, sPnpDevIDs.Length)
        cbWpdDevice.Items.Clear()
        Using IPDevManager As New PortableDeviceManager
            If IPDevManager.RefreshDeviceList = True Then
                Dim iDevCount As UInteger = IPDevManager.GetDeviceCount()
                If iDevCount > 0 Then
                    sPnpDevIDs = IPDevManager.GetDevices
                    For iItem = 0 To iDevCount - 1
                        Dim sFriendlyName As String = IPDevManager.GetDeviceFriendlyName(sPnpDevIDs(CInt(iItem)))
                        Dim sDescription As String = IPDevManager.GetDeviceDescription(sPnpDevIDs(CInt(iItem)))
                        If sFriendlyName = String.Empty OrElse sDescription = String.Empty Then
                            Using IPDevValues As New PortableDeviceValues
                                Using IPDev As New PortableDevice
                                    IPDevValues.SetStringValue(GetPropertyKey(PropertyKeys.WPD_CLIENT_NAME), "WPD-Demo")
                                    IPDevValues.SetStringValue(GetPropertyKey(PropertyKeys.WPD_CLIENT_MAJOR_VERSION), "1")
                                    IPDevValues.SetStringValue(GetPropertyKey(PropertyKeys.WPD_CLIENT_MINOR_VERSION), "0")
                                    IPDevValues.SetStringValue(GetPropertyKey(PropertyKeys.WPD_CLIENT_REVISION), "0")
                                    If IPDev.Open(sPnpDevIDs(CInt(iItem)), IPDevValues) = True Then
                                        Using IPDevContent As PortableDeviceContent = IPDev.Content
                                            sFriendlyName = GetPropertyStr(IPDevContent, WPD_DEVICE_OBJECT_ID,
                                                                        GetPropertyKey(PropertyKeys.WPD_DEVICE_FRIENDLY_NAME))

                                            sDescription = GetPropertyStr(IPDevContent, WPD_DEVICE_OBJECT_ID,
                                                                       GetPropertyKey(PropertyKeys.WPD_DEVICE_MODEL))

                                        End Using

                                        If IPDev.Cancel = True Then
                                            IPDev.Close()
                                        End If
                                    End If
                                End Using
                            End Using
                        End If
                        cbWpdDevice.Items.Add(sFriendlyName.Trim & " (" & sDescription.Trim & ")")
                    Next
                    cbWpdDevice.SelectedIndex = 0
                End If
            End If
        End Using
    End Sub

    Private Sub GetSupportedProperties(ByVal Content As PortableDeviceContent, ByVal objectId As String, ByVal oListBox As ListBox)
        Dim tKeys As WpdStruct.PROPERTYKEY
        oListBox.Items.Clear()
        Using currIPDevProperties As PortableDeviceProperties = Content.Properties
            Using currIPDevKeyCollection As PortableDeviceKeyCollection = currIPDevProperties.GetSupportedProperties(objectId)
                Dim iCount As UInteger = currIPDevKeyCollection.GetCount
                For iItem As UInteger = 0 To CUInt(iCount - 1)
                    tKeys = currIPDevKeyCollection.GetAt(iItem)
                    Using IPDevKeyCollection As New PortableDeviceKeyCollection
                        IPDevKeyCollection.Add(tKeys)
                        Using IPDevValues As PortableDeviceValues = currIPDevProperties.GetValues(objectId, IPDevKeyCollection)
                            Dim sDevValue As String = IPDevValues.GetStringValue(tKeys)
                            If sDevValue.Length <> 0 Then
                                If sDevValue.Substring(0, 1) = "{" Then
                                    oListBox.Items.Add(GetStringFromPropertyKey(tKeys) & " = " & GetGuidName(sDevValue))
                                Else
                                    oListBox.Items.Add(GetStringFromPropertyKey(tKeys) & " = " & sDevValue)
                                End If
                            Else
                                oListBox.Items.Add(GetStringFromPropertyKey(tKeys) & " = [not supported]") ' & sDevValue)
                            End If
                        End Using
                    End Using
                Next
            End Using
        End Using
    End Sub

    Private Sub GetSupportedCommands(ByVal Device As PortableDevice)
        Using IPDevCapabilities As PortableDeviceCapabilities = Device.Capabilities
            Using IPDevKeyCollection As PortableDeviceKeyCollection = IPDevCapabilities.GetSupportedCommands
                Dim iCount As UInteger = IPDevKeyCollection.GetCount
                For iItem As UInteger = 0 To CUInt(iCount - 1)
                    lbCommands.Items.Add(GetStringFromPropertyKey(IPDevKeyCollection.GetAt(iItem)))
                Next iItem
            End Using
        End Using
    End Sub

    Private Function GetSupportedCommand(ByVal Device As PortableDevice, ByRef Command As WpdStruct.PROPERTYKEY) As Boolean
        Dim bRet As Boolean = False
        Using IDevCapabilities As PortableDeviceCapabilities = Device.Capabilities
            Using IPDevKeyCollection As PortableDeviceKeyCollection = IDevCapabilities.GetSupportedCommands
                Dim iCount As UInteger = IPDevKeyCollection.GetCount
                For iItem As UInteger = 0 To CUInt(iCount - 1)
                    Dim tPropKey As WpdStruct.PROPERTYKEY = IPDevKeyCollection.GetAt(iItem)
                    If tPropKey.fmtid = Command.fmtid Then
                        If tPropKey.pid = Command.pid Then
                            bRet = True
                            Exit For
                        End If
                    End If
                Next iItem
            End Using
        End Using
        GetSupportedCommand = bRet
    End Function

    Private Sub ResetDevice(ByVal Device As PortableDevice)
        If GetSupportedCommand(Device, GetPropertyKey(PropertyKeys.WPD_COMMAND_COMMON_RESET_DEVICE)) = True Then
            ckResetDevice.Checked = True
        End If
    End Sub

    Private Sub ContentLocation(ByVal Device As PortableDevice)
        If GetSupportedCommand(Device, GetPropertyKey(PropertyKeys.WPD_COMMAND_DEVICE_HINTS_GET_CONTENT_LOCATION)) = True Then
            ckContentLocation.Checked = True
        End If
    End Sub

    Private Sub StorageEject(ByVal Device As PortableDevice)
        If GetSupportedCommand(Device, GetPropertyKey(PropertyKeys.WPD_COMMAND_STORAGE_EJECT)) = True Then
            ckStorageEject.Checked = True
        End If
    End Sub

    Private Sub StorageFormat(ByVal Device As PortableDevice)
        If GetSupportedCommand(Device, GetPropertyKey(PropertyKeys.WPD_COMMAND_STORAGE_FORMAT)) = True Then
            ckStorageFormat.Checked = True
        End If
    End Sub

    Private Sub StillImageCapture(ByVal Device As PortableDevice)
        If GetSupportedCommand(Device, GetPropertyKey(PropertyKeys.WPD_COMMAND_STILL_IMAGE_CAPTURE_INITIATE)) = True Then
            ckStillImageCapture.Checked = True
        End If
    End Sub

    Private Sub SmsSend(ByVal Device As PortableDevice)
        If GetSupportedCommand(Device, GetPropertyKey(PropertyKeys.WPD_COMMAND_SMS_SEND)) = True Then
            ckSmsSend.Checked = True
        End If
    End Sub

    Private Sub ListDeviceStorage(ByVal Content As PortableDeviceContent)
        Using IPDevProperties As PortableDeviceProperties = Content.Properties
            Using IPDevValues As New PortableDeviceValues
                Using IEPDevObjectIDs As EnumPortableDeviceObjectIDs = Content.EnumObjects(0, WPD_DEVICE_OBJECT_ID, IPDevValues)
                    Dim sObjID As String() = New String() {}
                    Dim uFetched As UInteger
                    If IEPDevObjectIDs.Next(1, sObjID, uFetched) = True Then
                        Do While uFetched <> 0
                            Using IPDevKeyCollection As PortableDeviceKeyCollection = New PortableDeviceKeyCollection
                                IPDevKeyCollection.Add(GetPropertyKey(PropertyKeys.WPD_FUNCTIONAL_OBJECT_PROPERTIES_V1))
                                IPDevKeyCollection.Add(GetPropertyKey(PropertyKeys.WPD_OBJECT_ID))
                                IPDevKeyCollection.Add(GetPropertyKey(PropertyKeys.WPD_OBJECT_NAME))
                                Using cDeviceValue As PortableDeviceValues = IPDevProperties.GetValues(sObjID(0), IPDevKeyCollection)

                                    If cDeviceValue.GetStringValue(GetPropertyKey(PropertyKeys.WPD_FUNCTIONAL_OBJECT_PROPERTIES_V1)) =
                                                                                               WPD_FUNCTIONAL_CATEGORY_STORAGE Then

                                        Dim sStorageID As String = cDeviceValue.GetStringValue(GetPropertyKey(PropertyKeys.WPD_OBJECT_ID))
                                        Dim sStorageName As String = cDeviceValue.GetStringValue(GetPropertyKey(PropertyKeys.WPD_OBJECT_NAME))
                                        cbStorage.Items.Add(sStorageID & " (" & sStorageName & ")")
                                        cbDrive.Items.Add(sStorageID & " (" & sStorageName & ")")
                                    End If
                                End Using
                            End Using
                            IEPDevObjectIDs.Next(1, sObjID, uFetched)

                        Loop
                    End If
                End Using
            End Using
        End Using

        If cbStorage.Items.Count > 0 Then cbStorage.SelectedIndex = 0
        If cbDrive.Items.Count > 0 Then cbDrive.SelectedIndex = 0

    End Sub

    Private Sub GetContentFolder(ByVal Content As PortableDeviceContent, ByVal ParentID As String, ByVal ParentName As String)
        tvDir.Nodes.Clear()
        lbFile.Items.Clear()
        Dim xNode As TreeNode = tvDir.Nodes.Add(ParentID, ParentName & " (" & ParentID & ")", 0)
        Application.DoEvents()
        EnumerateContent(Content, ParentID, 1, True, True)
        tvDir.SelectedNode = xNode
        xNode.Expand()

    End Sub

    Private Sub GetContentFiles(ByVal Content As PortableDeviceContent, ByVal ParentID As String)
        lbFile.Items.Clear()
        EnumerateContent(Content, ParentID, 0, False, False)
    End Sub

    Private Sub EnumerateContent(ByVal Content As PortableDeviceContent, ByVal ParentID As String,
                                 Optional ByVal Deep As Integer = 0, Optional ByVal FolderOnly As Boolean = False,
                                 Optional ByVal RecursiveFolder As Boolean = False)

        Using Properties As PortableDeviceProperties = Content.Properties
            Using Value As New PortableDeviceValues
                Using ObjectIDs As EnumPortableDeviceObjectIDs = Content.EnumObjects(0, ParentID, Value)
                    Dim sObjID As String() = New String() {}
                    Dim uFetched As UInteger
                    If ObjectIDs.Next(1, sObjID, uFetched) = True Then
                        Do While uFetched <> 0
                            Application.DoEvents()
                            Dim currentObject As WpdStruct.PDF = WrapObject(Properties, sObjID(0))
                            If currentObject.PDFF_Type = WPD_CONTENT_TYPE_FOLDER Then
                                If FolderOnly = True Then
                                    Dim xNode As TreeNode() = tvDir.Nodes.Find(sObjID(0), True)
                                    If xNode.Count = 0 Then
                                        xNode = tvDir.Nodes.Find(currentObject.PDFF_ParentId, True)
                                        xNode(0).Nodes.Add(currentObject.PDFF_ObjectId, currentObject.PDFF_OriginalName & " (" & currentObject.PDFF_ObjectId & ")", 1)
                                        xNode = tvDir.Nodes.Find(currentObject.PDFF_ObjectId, True)
                                        xNode(0).SelectedImageIndex = 1
                                    End If
                                End If
                            Else
                                If FolderOnly = False Then
                                    lbFile.Items.Add(currentObject.PDFF_OriginalName & " (" & currentObject.PDFF_ObjectId & ")")
                                End If
                            End If
                            If FolderOnly = True Then
                                If RecursiveFolder = True Then
                                    If currentObject.PDFF_Type = WPD_CONTENT_TYPE_FOLDER Then
                                        If Deep > 0 Then
                                            EnumerateContent(Content, sObjID(0), Deep - 1, FolderOnly, RecursiveFolder)
                                        End If
                                    End If
                                End If
                            End If
                            ObjectIDs.Next(1, sObjID, uFetched)
                        Loop
                    End If
                End Using
            End Using
        End Using
    End Sub

    Private Function WrapObject(ByVal Properties As PortableDeviceProperties, ByVal objectId As String) As WpdStruct.PDF
        Dim tRet As New WpdStruct.PDF
        Using Keys = New PortableDeviceKeyCollection
            Call Keys.Add(GetPropertyKey(PropertyKeys.WPD_OBJECT_ID))
            Call Keys.Add(GetPropertyKey(PropertyKeys.WPD_OBJECT_PARENT_ID))
            Call Keys.Add(GetPropertyKey(PropertyKeys.WPD_OBJECT_NAME))
            Call Keys.Add(GetPropertyKey(PropertyKeys.WPD_OBJECT_ORIGINAL_FILE_NAME))
            Call Keys.Add(GetPropertyKey(PropertyKeys.WPD_OBJECT_CONTENT_TYPE))
            Using Values = Properties.GetValues(objectId, Keys)
                If Values.GetCount > 0 Then
                    tRet.PDFF_ObjectId = Values.GetStringValue(GetPropertyKey(PropertyKeys.WPD_OBJECT_ID))
                    tRet.PDFF_ParentId = Values.GetStringValue(GetPropertyKey(PropertyKeys.WPD_OBJECT_PARENT_ID))
                    tRet.PDFF_Name = Values.GetStringValue(GetPropertyKey(PropertyKeys.WPD_OBJECT_NAME))
                    tRet.PDFF_OriginalName = Values.GetStringValue(GetPropertyKey(PropertyKeys.WPD_OBJECT_ORIGINAL_FILE_NAME))
                    tRet.PDFF_Type = Values.GetStringValue(GetPropertyKey(PropertyKeys.WPD_OBJECT_CONTENT_TYPE))
                End If
            End Using
        End Using
        Return tRet
    End Function

End Class

