﻿' Autor: Frank Schüler
' Datum: 01.01.2018
' Status: Komplett 2

Option Strict On
Option Explicit On

Imports System.Runtime.InteropServices

Public Class PortableDevice
    Implements IDisposable

    Private Const IID_IPortableDevice As String = "625E2DF8-6392-4CF0-9AD1-3CFA5F17775C"
    Private Const CLSID_PortableDevice As String = "728A21C5-3D9E-48D7-9810-864848F0F404"
    Private Const S_OK As Integer = 0

    Public _ifc As IPortableDevice

    Sub New()
        _ifc = DirectCast(Activator.CreateInstance(Type.GetTypeFromCLSID(New Guid(CLSID_PortableDevice))), IPortableDevice)
    End Sub

#Region "Interface IPortableDevice"
    ' https://msdn.microsoft.com/de-de/library/windows/desktop/dd319361(v=vs.85).aspx
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IPortableDevice)>
    Public Interface IPortableDevice

        Function Open(<[In], MarshalAs(UnmanagedType.LPWStr)> ByVal pszPnPDeviceID As String,
                      <[In]> ByVal pClientInfo As PortableDeviceValues.IPortableDeviceValues) As Integer

        Function SendCommand(<[In]> ByVal dwFlags As UInteger,
                             <[In], MarshalAs(UnmanagedType.Interface)> ByVal pParameters As PortableDeviceValues.IPortableDeviceValues,
                             <Out, MarshalAs(UnmanagedType.Interface)> ByRef ppResults As PortableDeviceValues.IPortableDeviceValues) As Integer

        Function Content(<Out, MarshalAs(UnmanagedType.Interface)> ByRef ppContent As PortableDeviceContent.IPortableDeviceContent) As Integer

        Function Capabilities(<Out, MarshalAs(UnmanagedType.Interface)> ByRef ppCapabilities As PortableDeviceCapabilities.IPortableDeviceCapabilities) As Integer

        Function Cancel() As Integer

        Function Close() As Integer

        Function Advise(<[In]> ByVal dwFlags As UInteger,
                        <[In], MarshalAs(UnmanagedType.Interface)> ByVal pCallback As PortableDeviceEventCallback.IPortableDeviceEventCallback,
                        <[In], MarshalAs(UnmanagedType.Interface)> ByVal pParameters As PortableDeviceValues.IPortableDeviceValues,
                        <Out, MarshalAs(UnmanagedType.LPWStr)> ByRef ppszCookie As String) As Integer

        Function Unadvise(<[In], MarshalAs(UnmanagedType.LPWStr)> ByVal ppszCookie As String) As Integer

        Function GetPnPDeviceID(<Out, MarshalAs(UnmanagedType.LPWStr)> ByRef ppszPnPDeviceID As String) As Integer

    End Interface
#End Region

#Region "Interface Helper"
    ' Opens a connection between the application and the device.
    Public Function Open(ByVal pszPnPDeviceID As String, ByVal pClientInfo As PortableDeviceValues) As Boolean
        Dim ret = _ifc.Open(pszPnPDeviceID, pClientInfo._ifc)
        If ret = S_OK Then
            Return True
        ElseIf ret <S_OK Then
            Throw New COMException("Open faild", ret)
        End If
        Return False
    End Function

    ' Sends a command to the device and retrieves the results synchronously.
    Public Function SendCommand(ByRef pParameters As PortableDeviceValues) As PortableDeviceValues
        Dim IPDevValues As New PortableDeviceValues
        Dim ret = _ifc.SendCommand(0, pParameters._ifc, IPDevValues._ifc)
        If ret = S_OK Then
            Return IPDevValues
        ElseIf ret < S_OK Then
            Throw New COMException("SendCommand faild", ret)
        End If
        Return Nothing
    End Function

    ' Retrieves an interface used to access objects on a device.
    Public Function Content() As PortableDeviceContent
        Dim IPDevContent As New PortableDeviceContent
        Dim ret = _ifc.Content(IPDevContent._ifc)
        If ret = S_OK Then
            Return IPDevContent
        ElseIf ret < S_OK Then
            Throw New COMException("Content faild", ret)
        End If
        Return Nothing
    End Function

    ' Retrieves an interface used to query the capabilities of a portable device.
    Public Function Capabilities() As PortableDeviceCapabilities
        Dim IPDevCapabilities As New PortableDeviceCapabilities
        Dim ret = _ifc.Capabilities(IPDevCapabilities._ifc)
        If ret = S_OK Then
            Return IPDevCapabilities
        ElseIf ret < S_OK Then
            Throw New COMException("Capabilities faild", ret)
        End If
        Return Nothing
    End Function

    ' Cancels a pending operation on this interface.
    Public Function Cancel() As Boolean
        Dim ret = _ifc.Cancel
        If ret = S_OK Then
            Return True
        ElseIf ret < S_OK Then
            Throw New COMException("Cancel faild", ret)
        End If
        Return False
    End Function

    ' Closes the connection with the device.
    Public Function Close() As Boolean
        Dim ret = _ifc.Close
        If ret = S_OK Then
            Return True
        ElseIf ret < S_OK Then
            Throw New COMException("Close faild", ret)
        End If
        Return False
    End Function

    ' Registers an application-defined callback that receives device events.
    Public Function Advise(ByVal dwFlags As UInteger,
                           ByRef pCallback As PortableDeviceEventCallback,
                           ByRef pParameters As PortableDeviceValues) As String
        Dim sRet As String = String.Empty
        Dim ret = _ifc.Advise(dwFlags, pCallback._ifc, pParameters._ifc, sRet)
        If ret = S_OK Then
            Return sRet
        ElseIf ret < S_OK Then
            Throw New COMException("Advise faild", ret)
        End If
        Return String.Empty
    End Function

    ' Unregisters a client from receiving callback notifications.
    Public Function Unadvise(ByVal pszCookie As String) As Boolean
        Dim ret = _ifc.Unadvise(pszCookie)
        If ret = S_OK Then
            Return True
        ElseIf ret < S_OK Then
            Throw New COMException("Unadvise faild", ret)
        End If
        Return False
    End Function

    ' Retrieves a Plug-and-Play (PnP) identifier used to open a device.
    Public Function GetPnPDeviceID() As String
        Dim sRet As String = String.Empty
        Dim ret = _ifc.GetPnPDeviceID(sRet)
        If ret = S_OK Then
            Return sRet
        ElseIf ret < S_OK Then
            Throw New COMException("GetPnPDeviceID faild", ret)
        End If
        Return String.Empty
    End Function
#End Region

#Region "IDisposable Support"
    Private disposedValue As Boolean

    Protected Overridable Sub Dispose(disposing As Boolean)
        If Not disposedValue Then
            If disposing Then
                ' TODO: verwalteten Zustand (verwaltete Objekte) entsorgen.
            End If

            If _ifc IsNot Nothing Then
                Marshal.ReleaseComObject(_ifc)
                _ifc = Nothing
            End If
        End If
        disposedValue = True
    End Sub

    Protected Overrides Sub Finalize()
        Dispose(False)
        MyBase.Finalize()
    End Sub

    Public Sub Dispose() Implements IDisposable.Dispose
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub
#End Region
End Class
