﻿' Autor: Frank Schüler
' Datum: 01.01.2018
' Status: Komplett 2

Option Strict On
Option Explicit On

Imports System.Runtime.InteropServices

Public Class PortableDeviceCapabilities
    Implements IDisposable

    Private Const IID_IPortableDeviceCapabilities As String = "2C8C6DBF-E3DC-4061-BECC-8542E810D126"
    Private Const CLSID_PortableDeviceCapabilities As String = ""
    Private Const S_OK As Integer = 0

    Public _ifc As IPortableDeviceCapabilities

#Region "Interface IPortableDeviceCapabilities"
    ' https://msdn.microsoft.com/en-us/library/windows/desktop/dd319362(v=vs.85).aspx
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IPortableDeviceCapabilities)>
    Public Interface IPortableDeviceCapabilities

        Function GetSupportedCommands(<Out, MarshalAs(UnmanagedType.Interface)> ByRef ppCommands As PortableDeviceKeyCollection.IPortableDeviceKeyCollection) As Integer

        Function GetCommandOptions(<[In], MarshalAs(UnmanagedType.Struct)> ByRef Command As WpdStruct.PROPERTYKEY,
                                   <Out, MarshalAs(UnmanagedType.Interface)> ByRef ppOptions As PortableDeviceValues.IPortableDeviceValues) As Integer

        Function GetFunctionalCategories(<Out, MarshalAs(UnmanagedType.Interface)> ByRef ppCategories As PortableDevicePropVariantCollection.IPortableDevicePropVariantCollection) As Integer

        Function GetFunctionalObjects(<[In], MarshalAs(UnmanagedType.Struct)> ByRef Category As Guid,
                                      <Out, MarshalAs(UnmanagedType.Interface)> ByRef ppObjectIDs As PortableDevicePropVariantCollection.IPortableDevicePropVariantCollection) As Integer

        Function GetSupportedContentTypes(<[In], MarshalAs(UnmanagedType.Struct)> ByRef Category As Guid,
                                         <Out, MarshalAs(UnmanagedType.Interface)> ByRef ppContentTypes As PortableDevicePropVariantCollection.IPortableDevicePropVariantCollection) As Integer

        Function GetSupportedFormats(<[In], MarshalAs(UnmanagedType.Struct)> ByRef ContentType As Guid,
                                    <Out, MarshalAs(UnmanagedType.Interface)> ByRef ppFormats As PortableDevicePropVariantCollection.IPortableDevicePropVariantCollection) As Integer

        Function GetSupportedFormatProperties(<[In], MarshalAs(UnmanagedType.Struct)> ByRef Format As Guid,
                                             <Out, MarshalAs(UnmanagedType.Interface)> ByRef ppKeys As PortableDeviceKeyCollection.IPortableDeviceKeyCollection) As Integer

        Function GetFixedPropertyAttributes(<[In], MarshalAs(UnmanagedType.Struct)> ByRef Format As Guid,
                                            <[In], MarshalAs(UnmanagedType.Struct)> ByRef key As WpdStruct.PROPERTYKEY,
                                            <Out, MarshalAs(UnmanagedType.Interface)> ByRef ppAttributes As PortableDeviceValues.IPortableDeviceValues) As Integer

        Function Cancel() As Integer

        Function GetSupportedEvents(<Out, MarshalAs(UnmanagedType.Interface)> ByRef ppEvents As PortableDevicePropVariantCollection.IPortableDevicePropVariantCollection) As Integer

        Function GetEventOptions(<[In], MarshalAs(UnmanagedType.Struct)> ByRef [Event] As Guid,
                                <Out, MarshalAs(UnmanagedType.Interface)> ByRef ppOptions As PortableDeviceValues.IPortableDeviceValues) As Integer

    End Interface
#End Region

#Region "Interface Helper"
    Function GetSupportedCommands() As PortableDeviceKeyCollection
        Dim ppCommands As New PortableDeviceKeyCollection
        Dim ret = _ifc.GetSupportedCommands(ppCommands._ifc)
        If ret = S_OK Then
            Return ppCommands
        ElseIf ret < S_OK Then
            Throw New COMException("GetSupportedCommands faild", ret)
        End If
        Return Nothing
    End Function

    Function GetCommandOptions(ByRef Command As WpdStruct.PROPERTYKEY) As PortableDeviceValues
        Dim ppOptions As New PortableDeviceValues
        Dim ret = _ifc.GetCommandOptions(Command, ppOptions._ifc)
        If ret = S_OK Then
            Return ppOptions
        ElseIf ret < S_OK Then
            Throw New COMException("GetCommandOptions faild", ret)
        End If
        Return Nothing
    End Function

    Function GetFunctionalCategories() As PortableDevicePropVariantCollection
        Dim ppCategories As New PortableDevicePropVariantCollection
        Dim ret = _ifc.GetFunctionalCategories(ppCategories._ifc)
        If ret = S_OK Then
            Return ppCategories
        ElseIf ret < S_OK Then
            Throw New COMException("GetFunctionalCategories faild", ret)
        End If
        Return Nothing
    End Function

    Function GetFunctionalObjects(ByRef Category As Guid) As PortableDevicePropVariantCollection
        Dim ppObjectIDs As New PortableDevicePropVariantCollection
        Dim ret = _ifc.GetFunctionalObjects(Category, ppObjectIDs._ifc)
        If ret = S_OK Then
            Return ppObjectIDs
        ElseIf ret < S_OK Then
            Throw New COMException("GetFunctionalObjects faild", ret)
        End If
        Return Nothing
    End Function

    Function GetSupportedContentTypes(ByRef Category As Guid) As PortableDevicePropVariantCollection
        Dim ppContentTypes As New PortableDevicePropVariantCollection
        Dim ret = _ifc.GetSupportedContentTypes(Category, ppContentTypes._ifc)
        If ret = S_OK Then
            Return ppContentTypes
        ElseIf ret < S_OK Then
            Throw New COMException("GetSupportedContentTypes faild", ret)
        End If
        Return Nothing
    End Function

    Function GetSupportedFormats(ByRef ContentType As Guid) As PortableDevicePropVariantCollection
        Dim ppFormats As New PortableDevicePropVariantCollection
        Dim ret = _ifc.GetSupportedFormats(ContentType, ppFormats._ifc)
        If ret = S_OK Then
            Return ppFormats
        ElseIf ret < S_OK Then
            Throw New COMException("GetSupportedFormats faild", ret)
        End If
        Return Nothing
    End Function

    Function GetSupportedFormatProperties(ByRef Format As Guid) As PortableDeviceKeyCollection
        Dim ppKeys As New PortableDeviceKeyCollection
        Dim ret = _ifc.GetSupportedFormatProperties(Format, ppKeys._ifc)
        If ret = S_OK Then
            Return ppKeys
        ElseIf ret < S_OK Then
            Throw New COMException("GetSupportedFormatProperties faild", ret)
        End If
        Return Nothing
    End Function

    Function GetFixedPropertyAttributes(ByRef Format As Guid, ByRef key As WpdStruct.PROPERTYKEY) As PortableDeviceValues
        Dim ppAttributes As New PortableDeviceValues
        Dim ret = _ifc.GetFixedPropertyAttributes(Format, key, ppAttributes._ifc)
        If ret = S_OK Then
            Return ppAttributes
        ElseIf ret < S_OK Then
            Throw New COMException("GetFixedPropertyAttributes faild", ret)
        End If
        Return Nothing
    End Function

    Function Cancel() As Boolean
        Dim ret = _ifc.Cancel
        If ret = S_OK Then
            Return True
        ElseIf ret < S_OK Then
            Throw New COMException("Cancel faild", ret)
        End If
        Return False
    End Function

    Function GetSupportedEvents() As PortableDevicePropVariantCollection
        Dim ppEvents As New PortableDevicePropVariantCollection
        Dim ret = _ifc.GetSupportedEvents(ppEvents._ifc)
        If ret = S_OK Then
            Return ppEvents
        ElseIf ret < S_OK Then
            Throw New COMException("GetSupportedEvents faild", ret)
        End If
        Return Nothing
    End Function

    Function GetEventOptions(ByRef [Event] As Guid) As PortableDeviceValues
        Dim ppOptions As New PortableDeviceValues
        Dim ret = _ifc.GetEventOptions([Event], ppOptions._ifc)
        If ret = S_OK Then
            Return ppOptions
        ElseIf ret < S_OK Then
            Throw New COMException("GetEventOptions faild", ret)
        End If
        Return Nothing
    End Function
#End Region

#Region "IDisposable Support"
    Private disposedValue As Boolean

    Protected Overridable Sub Dispose(disposing As Boolean)
        If Not disposedValue Then
            If disposing Then
                ' TODO: verwalteten Zustand (verwaltete Objekte) entsorgen.
            End If

            If _ifc IsNot Nothing Then
                Marshal.ReleaseComObject(_ifc)
                _ifc = Nothing
            End If
        End If
        disposedValue = True
    End Sub

    Protected Overrides Sub Finalize()
        Dispose(False)
        MyBase.Finalize()
    End Sub

    Public Sub Dispose() Implements IDisposable.Dispose
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub
#End Region

End Class

