﻿' Autor: Frank Schüler
' Datum: 01.01.2018
' Status: Komplett 2

Option Strict On
Option Explicit On

Imports System.Runtime.InteropServices

Public Class PortableDeviceContent2
    Implements IDisposable

    Private Const IID_IPortableDeviceContent2 As String = "9B4ADD96-F6BF-4034-8708-ECA72BF10554"
    Private Const CLSID_PortableDeviceContent2 As String = ""
    Private Const S_OK As Integer = 0

    Public _ifc As IPortableDeviceContent2

#Region "Interface IPortableDeviceContent2"
    ' https://msdn.microsoft.com/ru-RU/library/windows/desktop/dd388530
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IPortableDeviceContent2)>
    Public Interface IPortableDeviceContent2
        Inherits PortableDeviceContent.IPortableDeviceContent

        Shadows Function EnumObjects(<[In]> ByVal dwFlags As UInteger,
                                     <[In], MarshalAs(UnmanagedType.LPWStr)> ByVal pszParentObjectID As String,
                                     <[In], MarshalAs(UnmanagedType.Interface)> ByVal pFilter As PortableDeviceValues.IPortableDeviceValues,
                                     <Out, MarshalAs(UnmanagedType.Interface)> ByRef ppEnum As EnumPortableDeviceObjectIDs.IEnumPortableDeviceObjectIDs) As Integer

        Shadows Function Properties(<Out, MarshalAs(UnmanagedType.Interface)> ByRef ppProperties As PortableDeviceProperties.IPortableDeviceProperties) As Integer

        Shadows Function Transfer(<Out, MarshalAs(UnmanagedType.Interface)> ByRef ppResources As PortableDeviceResources.IPortableDeviceResources) As Integer

        Shadows Function CreateObjectWithPropertiesOnly(<[In], MarshalAs(UnmanagedType.Interface)> ByVal pValues As PortableDeviceValues.IPortableDeviceValues,
                                                        <Out, MarshalAs(UnmanagedType.LPWStr)> ByRef ppszObjectID As String) As Integer

        Shadows Function CreateObjectWithPropertiesAndData(<[In], MarshalAs(UnmanagedType.Interface)> ByVal pValues As PortableDeviceValues.IPortableDeviceValues,
                                                           <Out, MarshalAs(UnmanagedType.Interface)> ByRef ppData As Stream.IStream,
                                                           <[In], Out, MarshalAs(UnmanagedType.I2)> ByRef pdwOptimalWriteBufferSize As UInteger,
                                                           <[In], Out, MarshalAs(UnmanagedType.LPWStr)> ByRef ppszCookie As String) As Integer

        Shadows Function Delete(<[In]> ByVal dwOptions As Integer,
                                <[In], MarshalAs(UnmanagedType.Interface)> ByVal pObjectIDs As PortableDevicePropVariantCollection.IPortableDevicePropVariantCollection,
                                <[In], Out, MarshalAs(UnmanagedType.Interface)> ByRef ppResults As PortableDevicePropVariantCollection.IPortableDevicePropVariantCollection) As Integer

        Shadows Function GetObjectIDsFromPersistentUniqueIDs(<[In], MarshalAs(UnmanagedType.Interface)> ByVal pPersistentUniqueIDs As PortableDevicePropVariantCollection.IPortableDevicePropVariantCollection,
                                                             <Out, MarshalAs(UnmanagedType.Interface)> ByRef ppObjectIDs As PortableDevicePropVariantCollection.IPortableDevicePropVariantCollection) As Integer

        Shadows Function Cancel() As Integer

        Shadows Function Move(<[In], MarshalAs(UnmanagedType.Interface)> ByVal pObjectIDs As PortableDevicePropVariantCollection.IPortableDevicePropVariantCollection,
                              <[In], MarshalAs(UnmanagedType.LPWStr)> ByVal pszDestinationFolderObjectID As String,
                              <Out, MarshalAs(UnmanagedType.Interface)> ByRef ppResults As PortableDevicePropVariantCollection.IPortableDevicePropVariantCollection) As Integer

        Shadows Function Copy(<[In], MarshalAs(UnmanagedType.Interface)> ByVal pObjectIDs As PortableDevicePropVariantCollection.IPortableDevicePropVariantCollection,
                              <[In], MarshalAs(UnmanagedType.LPWStr)> ByVal pszDestinationFolderObjectID As String,
                              <Out, MarshalAs(UnmanagedType.Interface)> ByRef ppResults As PortableDevicePropVariantCollection.IPortableDevicePropVariantCollection) As Integer

        Function UpdateObjectWithPropertiesAndData(<[In], MarshalAs(UnmanagedType.LPWStr)> ByVal pszObjectID As String,
                                                   <[In], MarshalAs(UnmanagedType.Interface)> ByVal pProperties As PortableDeviceValues.IPortableDeviceValues,
                                                   <Out, MarshalAs(UnmanagedType.Interface)> ByRef ppData As Stream.IStream,
                                                   <[In], Out> ByRef pdwOptimalWriteBufferSize As UInteger) As Integer

    End Interface
#End Region

#Region "Interface Helper"
    ' Rretrieves an interface that is used to enumerate the immediate child objects of an object. 
    ' It has an Optional filter that can enumerate objects With specific properties.
    Function EnumObjects(ByVal dwFlags As UInteger,
                         ByVal pszParentObjectID As String,
                         ByRef pFilter As PortableDeviceValues) As EnumPortableDeviceObjectIDs
        Dim ppEnum As New EnumPortableDeviceObjectIDs
        Dim ret = _ifc.EnumObjects(dwFlags, pszParentObjectID, pFilter._ifc, ppEnum._ifc)
        If ret = S_OK Then
            Return ppEnum
        ElseIf ret < S_OK Then
            Throw New COMException("EnumObjects faild", ret)
        End If
        Return Nothing
    End Function

    ' Retrieves the interface that is required to get or set properties on an object on the device.
    Public Function Properties() As PortableDeviceProperties
        Dim ppProperties As New PortableDeviceProperties
        Dim ret = _ifc.Properties(ppProperties._ifc)
        If ret = S_OK Then
            Return ppProperties
        ElseIf ret < S_OK Then
            Throw New COMException("Properties faild", ret)
        End If
        Return Nothing
    End Function

    ' Retrieves an interface that is used to read from or write to the content data of an existing object resource.
    Function Transfer() As PortableDeviceResources
        Dim ppResources As New PortableDeviceResources
        Dim ret = _ifc.Transfer(ppResources._ifc)
        If ret = S_OK Then
            Return ppResources
        ElseIf ret < S_OK Then
            Throw New COMException("Transfer faild", ret)
        End If
        Return Nothing
    End Function

    ' Creates an object with only properties on the device.
    Function CreateObjectWithPropertiesOnly(ByRef pValues As PortableDeviceValues) As String
        Dim ppszObjectID As String = String.Empty
        Dim ret = _ifc.CreateObjectWithPropertiesOnly(pValues._ifc, ppszObjectID)
        If ret = S_OK Then
            Return ppszObjectID
        ElseIf ret < S_OK Then
            Throw New COMException("CreateObjectWithPropertiesOnly faild", ret)
        End If
        Return String.Empty
    End Function

    ' Creates an object with both properties and data on the device.
    Function CreateObjectWithPropertiesAndData(ByRef pValues As PortableDeviceValues,
                                               ByRef pdwOptimalWriteBufferSize As UInteger,
                                               ByRef ppszCookie As String) As Stream
        Dim ppData As New Stream
        Dim ret = _ifc.CreateObjectWithPropertiesAndData(pValues._ifc, ppData._ifc, pdwOptimalWriteBufferSize, ppszCookie)
        If ret = S_OK Then
            Return ppData
        ElseIf ret < S_OK Then
            Throw New COMException("CreateObjectWithPropertiesAndData faild", ret)
        End If
        Return Nothing
    End Function

    ' Deletes one or more objects from the device.
    Function Delete(ByVal dwOptions As WpdStruct.DELETE_OBJECT_OPTIONS,
                    ByRef pObjectIDs As PortableDevicePropVariantCollection) As PortableDevicePropVariantCollection
        Dim ppResults As New PortableDevicePropVariantCollection
        Dim ret = _ifc.Delete(dwOptions, pObjectIDs._ifc, ppResults._ifc)
        If ret = S_OK Then
            Return ppResults
        ElseIf ret < S_OK Then
            Throw New COMException("Delete faild", ret)
        End If
        Return Nothing
    End Function

    ' Retrieves the current object ID of one or more objects, given their persistent unique IDs (PUIDs).
    Function GetObjectIDsFromPersistentUniqueIDs(ByRef pPersistentUniqueIDs As PortableDevicePropVariantCollection) As PortableDevicePropVariantCollection
        Dim ppObjectIDs As New PortableDevicePropVariantCollection
        Dim ret = _ifc.GetObjectIDsFromPersistentUniqueIDs(pPersistentUniqueIDs._ifc, ppObjectIDs._ifc)
        If ret = S_OK Then
            Return ppObjectIDs
        ElseIf ret < S_OK Then
            Throw New COMException("GetObjectIDsFromPersistentUniqueIDs faild", ret)
        End If
        Return Nothing
    End Function

    ' Cancels a pending call on this interface.
    Function Cancel() As Boolean
        Dim ret = _ifc.Cancel
        If ret = S_OK Then
            Return True
        ElseIf ret < S_OK Then
            Throw New COMException("Cancel faild", ret)
        End If
        Return False
    End Function

    ' Moves one or more objects from one location on the device to another.
    Function Move(ByRef pObjectIDs As PortableDevicePropVariantCollection,
                  ByVal pszDestinationFolderObjectID As String) As PortableDevicePropVariantCollection
        Dim ppResults As New PortableDevicePropVariantCollection
        Dim ret = _ifc.Move(pObjectIDs._ifc, pszDestinationFolderObjectID, ppResults._ifc)
        If ret = S_OK Then
            Return ppResults
        ElseIf ret < S_OK Then
            Throw New COMException("Move faild", ret)
        End If
        Return Nothing
    End Function

    ' Copies objects from one location on a device to another.
    Function Copy(ByRef pObjectIDs As PortableDevicePropVariantCollection,
                  ByVal pszDestinationFolderObjectID As String) As PortableDevicePropVariantCollection
        Dim ppResults As New PortableDevicePropVariantCollection
        Dim ret = _ifc.Move(pObjectIDs._ifc, pszDestinationFolderObjectID, ppResults._ifc)
        If ret = S_OK Then
            Return ppResults
        ElseIf ret < S_OK Then
            Throw New COMException("Copy faild", ret)
        End If
        Return Nothing
    End Function

    ' Updates an object with properties and data found on the device.
    Function UpdateObjectWithPropertiesAndData(ByVal pszObjectID As String,
                                               ByRef pProperties As PortableDeviceValues,
                                               ByRef pdwOptimalWriteBufferSize As UInteger) As Stream
        Dim ppData As New Stream
        Dim ret = _ifc.UpdateObjectWithPropertiesAndData(pszObjectID, pProperties._ifc, ppData._ifc, pdwOptimalWriteBufferSize)
        If ret = S_OK Then
            Return ppData
        ElseIf ret < S_OK Then
            Throw New COMException("UpdateObjectWithPropertiesAndData faild", ret)
        End If
        Return Nothing
    End Function

#End Region

#Region "IDisposable Support"
    Private disposedValue As Boolean

    Protected Overridable Sub Dispose(disposing As Boolean)
        If Not disposedValue Then
            If disposing Then
                ' TODO: verwalteten Zustand (verwaltete Objekte) entsorgen.
            End If

            If _ifc IsNot Nothing Then
                Marshal.ReleaseComObject(_ifc)
                _ifc = Nothing
            End If
        End If
        disposedValue = True
    End Sub

    Protected Overrides Sub Finalize()
        Dispose(False)
        MyBase.Finalize()
    End Sub

    Public Sub Dispose() Implements IDisposable.Dispose
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub
#End Region

End Class
