﻿' Autor: Frank Schüler
' Datum: 01.01.2018
' Status: Komplett 2

Option Strict On
Option Explicit On

Imports System.Runtime.InteropServices

Public Class PortableDeviceKeyCollection
    Implements IDisposable

    Private Const IID_IPortableDeviceKeyCollection As String = "DADA2357-E0AD-492E-98DB-DD61C53BA353"
    Private Const CLSID_PortableDeviceKeyCollection As String = "DE2D022D-2480-43BE-97F0-D1FA2CF98F4F"
    Private Const S_OK As Integer = 0

    Public _ifc As IPortableDeviceKeyCollection

    Sub New()
        _ifc = DirectCast(Activator.CreateInstance(Type.GetTypeFromCLSID(New Guid(CLSID_PortableDeviceKeyCollection))), IPortableDeviceKeyCollection)
    End Sub

#Region "Interface IPortableDeviceKeyCollection"
    ' https://msdn.microsoft.com/en-us/library/windows/desktop/dd388547(v=vs.85).aspx
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IPortableDeviceKeyCollection)>
    Public Interface IPortableDeviceKeyCollection

        Function GetCount(<[In], Out> ByRef pcElems As UInteger) As Integer

        Function GetAt(<[In]> ByVal dwIndex As UInteger,
                       <Out, MarshalAs(UnmanagedType.Struct)> ByRef pKey As WpdStruct.PROPERTYKEY) As Integer

        Function Add(<[In], MarshalAs(UnmanagedType.Struct)> ByRef Key As WpdStruct.PROPERTYKEY) As Integer

        Function Clear() As Integer

        Function RemoveAt(<[In]> ByVal dwIndex As UInteger) As Integer

    End Interface
#End Region

#Region "Interface Helper"
    ' Retrieves the number of keys in this collection.
    Public Function GetCount() As UInteger
        Dim pcElems As UInteger = 0
        Dim ret = _ifc.GetCount(pcElems)
        If ret = S_OK Then
            Return pcElems
        ElseIf ret < S_OK Then
            Throw New COMException("GetCount faild", ret)
        End If
        Return 0
    End Function

    ' Retrieves a PROPERTYKEY from the collection by index.
    Public Function GetAt(ByVal dwIndex As UInteger) As WpdStruct.PROPERTYKEY
        Dim pKey As WpdStruct.PROPERTYKEY
        Dim ret = _ifc.GetAt(dwIndex, pKey)
        If ret = S_OK Then
            Return pKey
        ElseIf ret < S_OK Then
            Throw New COMException("Add faild", ret)
        End If
        Return Nothing
    End Function

    ' Adds a property key to the collection.
    Public Function Add(ByRef Key As WpdStruct.PROPERTYKEY) As Boolean
        Dim ret = _ifc.Add(Key)
        If ret = S_OK Then
            Return True
        ElseIf ret < S_OK Then
            Throw New COMException("Add faild", ret)
        End If
        Return False
    End Function

    ' Deletes all items from the collection.
    Public Function Clear() As Boolean
        Dim ret = _ifc.Clear
        If ret = S_OK Then
            Return True
        ElseIf ret < S_OK Then
            Throw New COMException("Clear faild", ret)
        End If
        Return False
    End Function

    ' Removes the element stored at the location specified by the given index.
    Public Function RemoveAt(ByVal dwIndex As UInteger) As Boolean
        Dim ret = _ifc.RemoveAt(dwIndex)
        If ret = S_OK Then
            Return True
        ElseIf ret < S_OK Then
            Throw New COMException("RemoveAt faild", ret)
        End If
        Return False
    End Function
#End Region

#Region "IDisposable Support"
    Private disposedValue As Boolean

    Protected Overridable Sub Dispose(disposing As Boolean)
        If Not disposedValue Then
            If disposing Then
                ' TODO: verwalteten Zustand (verwaltete Objekte) entsorgen.
            End If

            If _ifc IsNot Nothing Then
                Marshal.ReleaseComObject(_ifc)
                _ifc = Nothing
            End If
        End If
        disposedValue = True
    End Sub

    Protected Overrides Sub Finalize()
        Dispose(False)
        MyBase.Finalize()
    End Sub

    Public Sub Dispose() Implements IDisposable.Dispose
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub
#End Region

End Class
