﻿' Autor: Frank Schüler
' Datum: 01.01.2018
' Status: Komplett 2

Option Strict On
Option Explicit On

Imports System.Text
Imports System.Runtime.InteropServices

Public Class PortableDeviceManager
    Implements IDisposable

    Private Const IID_IPortableDeviceManager As String = "A1567595-4C2F-4574-A6FA-ECEF917B9A40"
    Private Const CLSID_PortableDeviceManager As String = "0AF10CEC-2ECD-4B92-9581-34F6AE0637F3"
    Private Const S_OK As Integer = 0

    Public _ifc As IPortableDeviceManager

    Sub New()
        _ifc = DirectCast(Activator.CreateInstance(Type.GetTypeFromCLSID(New Guid(CLSID_PortableDeviceManager))), IPortableDeviceManager)
    End Sub

#Region "Interface IPortableDeviceManager"

    ' https://msdn.microsoft.com/de-de/library/windows/desktop/dd388688(v=vs.85).aspx
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IPortableDeviceManager)>
    Public Interface IPortableDeviceManager

        Function GetDevices(<[In], Out, MarshalAs(UnmanagedType.LPArray, ArraySubType:=UnmanagedType.LPWStr, SizeParamIndex:=1)> ByVal pPnPDeviceIDs As String(),
                           <[In], Out> ByRef pcPnPDeviceIDs As UInteger) As Integer

        Function RefreshDeviceList() As Integer

        Function GetDeviceFriendlyName(<[In], MarshalAs(UnmanagedType.LPWStr)> ByVal pszPnPDeviceID As String,
                                      <[In], Out, MarshalAs(UnmanagedType.LPWStr)> ByVal pDeviceFriendlyName As StringBuilder,
                                      <[In], Out> ByRef pcchDeviceFriendlyName As UInteger) As Integer

        Function GetDeviceDescription(<[In], MarshalAs(UnmanagedType.LPWStr)> ByVal pszPnPDeviceID As String,
                                     <[In], Out, MarshalAs(UnmanagedType.LPWStr)> ByVal pDeviceDescription As StringBuilder,
                                     <[In], Out> ByRef pcchDeviceDescription As UInteger) As Integer

        Function GetDeviceManufacturer(<[In], MarshalAs(UnmanagedType.LPWStr)> ByVal pszPnPDeviceID As String,
                                      <[In], Out, MarshalAs(UnmanagedType.LPWStr)> ByVal pDeviceManufacturer As StringBuilder,
                                      <[In], Out> ByRef pcchDeviceManufacturer As UInteger) As Integer

        Function GetDeviceProperty(<[In], MarshalAs(UnmanagedType.LPWStr)> ByVal pszPnPDeviceID As String,
                                  <[In], MarshalAs(UnmanagedType.LPWStr)> ByVal pszDevicePropertyName As String,
                                  <[In], Out, MarshalAs(UnmanagedType.LPArray, ArraySubType:=UnmanagedType.ByValArray, SizeParamIndex:=1)> ByVal pData As Byte(),
                                  <[In], Out> ByRef pcbData As UInteger,
                                  <[In], Out> ByRef pdwType As UInteger) As Integer

        Function GetPrivateDevices(<[In], Out, MarshalAs(UnmanagedType.LPArray, ArraySubType:=UnmanagedType.LPWStr, SizeParamIndex:=1)> ByVal pPnPDeviceIDs As String(),
                                  <[In], Out> ByRef pcPnPDeviceIDs As UInteger) As Integer

    End Interface
#End Region

#Region "Interface Helper"
    ' Retrieves the count of portable devices connected to the computer
    Public Function GetDeviceCount() As UInteger
        Dim iDevCount As UInteger
        Dim ret = _ifc.GetDevices(Nothing, iDevCount)
        If ret = S_OK Then
            Return iDevCount
        ElseIf ret < S_OK Then
            Throw New COMException("GetDevices faild", ret)
        End If
        Return 0
    End Function

    ' Retrieves a list of portable devices connected to the computer
    Public Function GetDevices() As String()
        Dim iDevCount As UInteger = GetDeviceCount()
        If iDevCount > 0 Then
            Dim sPnpDeviceID = New String(CInt(iDevCount - 1)) {}
            Dim ret = _ifc.GetDevices(sPnpDeviceID, iDevCount)
            If ret = S_OK Then
                Return sPnpDeviceID
            ElseIf ret < S_OK Then
                Throw New COMException("GetDevices faild", ret)
            End If
        Else
            Return New String(-1) {}
        End If
        Return Nothing
    End Function

    ' Refreshes the list of portable devices that are connected to the computer
    Public Function RefreshDeviceList() As Boolean
        Dim ret = _ifc.RefreshDeviceList
        If ret = S_OK Then
            Return True
        ElseIf ret < S_OK Then
            Throw New COMException("RefreshDeviceList faild", ret)
        End If
        Return False
    End Function

    ' Retrieves the user-friendly name for the device
    Public Function GetDeviceFriendlyName(ByVal PnPDeviceID As String) As String
        Dim iCharCount As UInteger
        Dim ret = _ifc.GetDeviceFriendlyName(PnPDeviceID, Nothing, iCharCount)
        If ret = S_OK Then
            If iCharCount > 0 Then
                Dim sDeviceFriendlyName As StringBuilder = New StringBuilder(Char.MinValue, CInt(iCharCount - 1))
                ret = _ifc.GetDeviceFriendlyName(PnPDeviceID, sDeviceFriendlyName, iCharCount)
                If ret = S_OK Then
                    Return sDeviceFriendlyName.ToString
                ElseIf ret < S_OK Then
                    Throw New COMException("GetDeviceFriendlyName faild", ret)
                End If
            End If
        ElseIf ret < S_OK Then
            Throw New COMException("GetDeviceFriendlyName faild", ret)
        End If
        Return String.Empty
    End Function

    ' Retrieves the description of a device
    Public Function GetDeviceDescription(ByVal PnPDeviceID As String) As String
        Dim iCharCount As UInteger
        Dim ret = _ifc.GetDeviceDescription(PnPDeviceID, Nothing, iCharCount)
        If ret = S_OK Then
            If iCharCount > 0 Then
                Dim sDeviceDescription As StringBuilder = New StringBuilder(Char.MinValue, CInt(iCharCount - 1))
                ret = _ifc.GetDeviceDescription(PnPDeviceID, sDeviceDescription, iCharCount)
                If ret = S_OK Then
                    Return sDeviceDescription.ToString
                ElseIf ret < S_OK Then
                    Throw New COMException("GetDeviceDescription faild", ret)
                End If
            End If
        ElseIf ret < S_OK Then
            Throw New COMException("GetDeviceDescription faild", ret)
        End If
        Return String.Empty
    End Function

    ' Retrieves the name of the device manufacturer
    Public Function GetDeviceManufacturer(ByVal PnPDeviceID As String) As String
        Dim iCharCount As UInteger
        Dim ret = _ifc.GetDeviceManufacturer(PnPDeviceID, Nothing, iCharCount)
        If ret = S_OK Then
            If iCharCount > 0 Then
                Dim sDeviceDescription As StringBuilder = New StringBuilder(Char.MinValue, CInt(iCharCount - 1))
                ret = _ifc.GetDeviceManufacturer(PnPDeviceID, sDeviceDescription, iCharCount)
                If ret = S_OK Then
                    Return sDeviceDescription.ToString
                ElseIf ret < S_OK Then
                    Throw New COMException("GetDeviceManufacturer faild", ret)
                End If
            End If
        ElseIf ret < S_OK Then
            Throw New COMException("GetDeviceManufacturer faild", ret)
        End If
        Return String.Empty
    End Function

    ' Retrieves a property value stored by the device on the computer
    Public Function GetDeviceProperty(ByVal sPnpDeviceID As String, ByVal sDevicePropertyName As String) As String
        Dim iData As UInteger
        Dim iType As UInteger
        Dim ret = _ifc.GetDeviceProperty(sPnpDeviceID, sDevicePropertyName, Nothing, iData, iType)
        If ret = S_OK Then
            If iData > 0 Then
                Dim bData As Byte() = New Byte(CInt(iData - 1)) {}
                ret = _ifc.GetDeviceProperty(sPnpDeviceID, sDevicePropertyName, bData, iData, iType)
                If ret = S_OK Then
                ElseIf ret < S_OK Then
                    Throw New COMException("GetDeviceProperty faild", ret)
                End If
            End If
        ElseIf ret < S_OK Then
            Throw New COMException("GetDeviceProperty faild", ret)
        End If
        Return String.Empty
    End Function

    ' Retrieves the count of private portable devices connected to the computer
    Public Function GetPrivateDeviceCount() As UInteger
        Dim iDevCount As UInteger
        Dim ret = _ifc.GetPrivateDevices(Nothing, iDevCount)
        If ret = S_OK Then
            Return iDevCount
        ElseIf ret < S_OK Then
            Throw New COMException("GetPrivateDevices faild", ret)
        End If
        Return 0
    End Function

    ' Retrieves a list of private portable devices connected to the computer
    Public Function GetPrivateDevices() As String()
        Dim iDevCount As UInteger = GetPrivateDeviceCount()
        If iDevCount > 0 Then
            Dim sPnpDeviceID = New String(CInt(iDevCount - 1)) {}
            Dim ret = _ifc.GetPrivateDevices(sPnpDeviceID, iDevCount)
            If ret = S_OK Then
                Return sPnpDeviceID
            ElseIf ret < S_OK Then
                Throw New COMException("GetPrivateDevices faild", ret)
            End If
        Else
            Return New String(-1) {}
        End If
        Return Nothing
    End Function
#End Region

#Region "IDisposable Support"
    Private disposedValue As Boolean

    Protected Overridable Sub Dispose(disposing As Boolean)
        If Not disposedValue Then
            If disposing Then
                ' TODO: verwalteten Zustand (verwaltete Objekte) entsorgen.
            End If

            If _ifc IsNot Nothing Then
                Marshal.ReleaseComObject(_ifc)
                _ifc = Nothing
            End If
        End If
        disposedValue = True
    End Sub

    Protected Overrides Sub Finalize()
        Dispose(False)
        MyBase.Finalize()
    End Sub

    Public Sub Dispose() Implements IDisposable.Dispose
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub
#End Region

End Class
