﻿' Autor: Frank Schüler
' Datum: 01.01.2018
' Status: Komplett 2

Option Strict On
Option Explicit On

Imports System.Runtime.InteropServices

Public Class PortableDevicePropVariantCollection
    Implements IDisposable

    Private Const IID_IPortableDevicePropVariantCollection As String = "89B2E422-4F1B-4316-BCEF-A44AFEA83EB3"
    Private Const CLSID_PortableDevicePropVariantCollection As String = "08A99E2F-6D6D-4B80-AF5A-BAF2BCBE4CB9"
    Private Const S_OK As Integer = 0

    Public _ifc As IPortableDevicePropVariantCollection

    Sub New()
        _ifc = DirectCast(Activator.CreateInstance(Type.GetTypeFromCLSID(New Guid(CLSID_PortableDevicePropVariantCollection))), IPortableDevicePropVariantCollection)
    End Sub

#Region "Interface IPortableDevicePropVariantCollection"
    ' https://msdn.microsoft.com/en-us/library/windows/desktop/dd388719(v=vs.85).aspx
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IPortableDevicePropVariantCollection)>
    Public Interface IPortableDevicePropVariantCollection
        Function GetCount(<[In], Out> ByRef pcElems As UInteger) As Integer

        Function GetAt(<[In]> ByVal dwIndex As UInteger, <[In], Out, MarshalAs(UnmanagedType.Struct)> ByRef pValue As WpdStruct.PROPVARIANT) As Integer

        Function Add(<[In], MarshalAs(UnmanagedType.Struct)> ByRef pValue As WpdStruct.PROPVARIANT) As Integer

        Function [GetType](<Out> ByRef pvt As UShort) As Integer

        Function ChangeType(<[In]> ByVal vt As UShort) As Integer

        Function Clear() As Integer

        Function RemoveAt(<[In]> ByVal dwIndex As UInteger) As Integer

    End Interface
#End Region

#Region "Interface Helper"

    ' Retrieves the number of items in this collection.
    Function GetCount() As UInteger
        Dim pcElems As UInteger
        Dim ret = _ifc.GetCount(pcElems)
        If ret = S_OK Then
            Return pcElems
        ElseIf ret < S_OK Then
            Throw New COMException("GetCount faild", ret)
        End If
        Return 0
    End Function

    ' Retrieves an item from the collection by a zero-based index.
    Function GetAt(ByVal dwIndex As UInteger, ByRef pValue As WpdStruct.PROPVARIANT) As Boolean
        Dim ret = _ifc.GetAt(dwIndex, pValue)
        If ret = S_OK Then
            Return True
        ElseIf ret < S_OK Then
            Throw New COMException("GetAt faild", ret)
        End If
        Return False
    End Function

    ' Adds an item to the collection.
    Function Add(ByRef pValue As WpdStruct.PROPVARIANT) As Boolean
        Dim ret = _ifc.Add(pValue)
        If ret = S_OK Then
            Return True
        ElseIf ret < S_OK Then
            Throw New COMException("Add faild", ret)
        End If
        Return False
    End Function

    ' Retrieves the data type of the items in the collection.
    Function [GetType]() As UShort
        Dim pvt As UShort
        Dim ret = _ifc.GetType(pvt)
        If ret = S_OK Then
            Return pvt
        ElseIf ret < S_OK Then
            Throw New COMException("GetType faild", ret)
        End If
        Return 0
    End Function

    ' Converts all items in the collection to the specified VARTYPE.
    Function ChangeType(ByVal vt As UShort) As Boolean
        Dim ret = _ifc.ChangeType(vt)
        If ret = S_OK Then
            Return True
        ElseIf ret < S_OK Then
            Throw New COMException("ChangeType faild", ret)
        End If
        Return False
    End Function

    ' Frees, and then removes, all items from the collection.
    Function Clear() As Boolean
        Dim ret = _ifc.Clear
        If ret = S_OK Then
            Return True
        ElseIf ret < S_OK Then
            Throw New COMException("Clear faild", ret)
        End If
        Return False
    End Function

    ' Removes the element stored at the location specified by the given index.
    Function RemoveAt(ByVal dwIndex As UInteger) As Boolean
        Dim ret = _ifc.RemoveAt(dwIndex)
        If ret = S_OK Then
            Return True
        ElseIf ret < S_OK Then
            Throw New COMException("RemoveAt faild", ret)
        End If
        Return False
    End Function
#End Region

#Region "IDisposable Support"
    Private disposedValue As Boolean

    Protected Overridable Sub Dispose(disposing As Boolean)
        If Not disposedValue Then
            If disposing Then
                ' TODO: verwalteten Zustand (verwaltete Objekte) entsorgen.
            End If

            If _ifc IsNot Nothing Then
                Marshal.ReleaseComObject(_ifc)
                _ifc = Nothing
            End If
        End If
        disposedValue = True
    End Sub

    Protected Overrides Sub Finalize()
        Dispose(False)
        MyBase.Finalize()
    End Sub

    Public Sub Dispose() Implements IDisposable.Dispose
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub
#End Region

End Class
