﻿' Autor: Frank Schüler
' Datum: 01.01.2018
' Status: Komplett 2

Option Strict On
Option Explicit On

Imports System.Runtime.InteropServices

Public Class PortableDeviceProperties
    Implements IDisposable

    Private Const IID_IPortableDeviceProperties As String = "7F6D695C-03DF-4439-A809-59266BEEE3A6"
    Private Const CLSID_PortableDeviceProperties As String = ""
    Private Const S_OK As Integer = 0

    Public _ifc As IPortableDeviceProperties

#Region "Interface IPortableDeviceProperties"
    ' https://msdn.microsoft.com/de-de/library/windows/desktop/dd388696.aspx
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IPortableDeviceProperties)>
    Public Interface IPortableDeviceProperties

        Function GetSupportedProperties(<[In], MarshalAs(UnmanagedType.LPWStr)> ByVal pszObjectID As String,
                                        <Out, MarshalAs(UnmanagedType.Interface)> ByRef ppKeys As PortableDeviceKeyCollection.IPortableDeviceKeyCollection) As Integer

        Function GetPropertyAttributes(<[In], MarshalAs(UnmanagedType.LPWStr)> ByVal pszObjectID As String,
                                       <[In], MarshalAs(UnmanagedType.Struct)> ByRef Key As WpdStruct.PROPERTYKEY,
                                       <Out, MarshalAs(UnmanagedType.Interface)> ByRef ppValues As PortableDeviceValues.IPortableDeviceValues) As Integer

        Function GetValues(<[In], MarshalAs(UnmanagedType.LPWStr)> ByVal pszObjectID As String,
                           <[In], MarshalAs(UnmanagedType.Interface)> ByVal pKeys As PortableDeviceKeyCollection.IPortableDeviceKeyCollection,
                           <Out, MarshalAs(UnmanagedType.Interface)> ByRef ppValues As PortableDeviceValues.IPortableDeviceValues) As Integer

        Function SetValues(<[In], MarshalAs(UnmanagedType.LPWStr)> ByVal pszObjectID As String,
                           <[In], MarshalAs(UnmanagedType.Interface)> ByVal pValues As PortableDeviceValues.IPortableDeviceValues,
                           <Out, MarshalAs(UnmanagedType.Interface)> ByRef ppValues As PortableDeviceValues.IPortableDeviceValues) As Integer


        Function Delete(<[In], MarshalAs(UnmanagedType.LPWStr)> ByVal pszObjectID As String,
                        <[In], MarshalAs(UnmanagedType.Interface)> ByRef pKeys As PortableDeviceKeyCollection.IPortableDeviceKeyCollection) As Integer

        Function Cancel() As Integer

    End Interface
#End Region

#Region "Interface Helper"
    ' Retrieves a list of properties that a specified object supports.
    Function GetSupportedProperties(ByVal pszObjectID As String) As PortableDeviceKeyCollection
        Dim ppKeys As New PortableDeviceKeyCollection
        Dim ret = _ifc.GetSupportedProperties(pszObjectID, ppKeys._ifc)
        If ret = S_OK Then
            Return ppKeys
        ElseIf ret < S_OK Then
            Throw New COMException("GetSupportedProperties faild", ret)
        End If
        Return Nothing
    End Function

    ' Retrieves attributes of a specified object property on a device.
    Function GetPropertyAttributes(ByVal pszObjectID As String, ByRef Key As WpdStruct.PROPERTYKEY) As PortableDeviceValues
        Dim ppValues As New PortableDeviceValues
        Dim ret = _ifc.GetPropertyAttributes(pszObjectID, Key, ppValues._ifc)
        If ret = S_OK Then
            Return ppValues
        ElseIf ret < S_OK Then
            Throw New COMException("GetPropertyAttributes faild", ret)
        End If
        Return Nothing
    End Function

    ' Retrieves a list of specified properties from a specified object on a device.
    Public Function GetValues(ByVal pszObjectID As String, ByRef pKeys As PortableDeviceKeyCollection) As PortableDeviceValues
        Dim IPDevValues As New PortableDeviceValues
        Dim ret = _ifc.GetValues(pszObjectID, pKeys._ifc, IPDevValues._ifc)
        If ret = S_OK Then
            Return IPDevValues
        ElseIf ret < S_OK Then
            Throw New COMException("GetValues faild", ret)
        End If
        Return Nothing
    End Function

    ' Adds or modifies one or more properties on a specified object on a device.
    Function SetValues(ByVal pszObjectID As String, ByVal pValues As PortableDeviceValues) As PortableDeviceValues
        Dim ppValues As New PortableDeviceValues
        Dim ret = _ifc.SetValues(pszObjectID, pValues._ifc, ppValues._ifc)
        If ret = S_OK Then
            Return ppValues
        ElseIf ret < S_OK Then
            Throw New COMException("SetValues faild", ret)
        End If
        Return Nothing
    End Function

    ' Deletes specified properties from a specified object on a device.
    Function Delete(ByVal pszObjectID As String, ByRef pKeys As PortableDeviceKeyCollection) As Boolean
        Dim ret = _ifc.GetSupportedProperties(pszObjectID, pKeys._ifc)
        If ret = S_OK Then
            Return True
        ElseIf ret < S_OK Then
            Throw New COMException("Delete faild", ret)
        End If
        Return False
    End Function

    ' Cancels a pending call on this interface.
    Function Cancel() As Boolean
        Dim ret = _ifc.Cancel
        If ret = S_OK Then
            Return True
        ElseIf ret < S_OK Then
            Throw New COMException("Cancel faild", ret)
        End If
        Return False
    End Function
#End Region

#Region "IDisposable Support"
    Private disposedValue As Boolean

    Protected Overridable Sub Dispose(disposing As Boolean)
        If Not disposedValue Then
            If disposing Then
                ' TODO: verwalteten Zustand (verwaltete Objekte) entsorgen.
            End If

            If _ifc IsNot Nothing Then
                Marshal.ReleaseComObject(_ifc)
                _ifc = Nothing
            End If
        End If
        disposedValue = True
    End Sub

    Protected Overrides Sub Finalize()
        Dispose(False)
        MyBase.Finalize()
    End Sub

    Public Sub Dispose() Implements IDisposable.Dispose
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub
#End Region

End Class
