﻿' Autor: Frank Schüler
' Datum: 01.01.2018
' Status: Komplett 2

Option Strict On
Option Explicit On

Imports System.Runtime.InteropServices

Public Class PortableDeviceResources
    Implements IDisposable

    Private Const IID_IPortableDeviceResources As String = "FD8878AC-D841-4D17-891C-E6829CDB6934"
    Private Const CLSID_PortableDeviceResources As String = ""
    Private Const S_OK As Integer = 0

    Public _ifc As IPortableDeviceResources

#Region "Interface IPortableDeviceResources"
    ' https://msdn.microsoft.com/en-us/library/windows/desktop/dd388733(v=vs.85).aspx
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IPortableDeviceResources)>
    Public Interface IPortableDeviceResources
        Function GetSupportedResources(<[In], MarshalAs(UnmanagedType.LPWStr)> ByVal pszObjectID As String,
                                       <Out, MarshalAs(UnmanagedType.Interface)> ByRef ppKeys As PortableDeviceKeyCollection.IPortableDeviceKeyCollection) As Integer

        Function GetResourceAttributes(<[In], MarshalAs(UnmanagedType.LPWStr)> ByVal pszObjectID As String,
                                       <[In], MarshalAs(UnmanagedType.Struct)> ByRef Key As WpdStruct.PROPERTYKEY,
                                       <Out, MarshalAs(UnmanagedType.Interface)> ByRef ppResourceAttributes As PortableDeviceValues.IPortableDeviceValues) As Integer

        Function GetStream(<[In], MarshalAs(UnmanagedType.LPWStr)> ByVal pszObjectID As String,
                           <[In], MarshalAs(UnmanagedType.Struct)> ByRef Key As WpdStruct.PROPERTYKEY,
                           <[In]> ByVal dwMode As Integer,
                           <[In], Out> ByRef pdwOptimalBufferSize As UInteger,
                           <Out, MarshalAs(UnmanagedType.Interface)> ByRef ppStream As Stream.IStream) As Integer

        Function Delete(<[In], MarshalAs(UnmanagedType.LPWStr)> ByVal pszObjectID As String,
                        <[In], MarshalAs(UnmanagedType.Interface)> ByVal pKeys As PortableDeviceKeyCollection.IPortableDeviceKeyCollection) As Integer

        Function Cancel() As Integer

    End Interface
#End Region

#Region "Interface Helper"
    'Retrieves a list of resources supported by a specific object.
    Function GetSupportedResources(ByVal pszObjectID As String) As PortableDeviceKeyCollection
        Dim ppKeys As New PortableDeviceKeyCollection
        Dim ret = _ifc.GetSupportedResources(pszObjectID, ppKeys._ifc)
        If ret = S_OK Then
            Return ppKeys
        ElseIf ret < S_OK Then
            Throw New COMException("GetSupportedResources faild", ret)
        End If
        Return Nothing
    End Function

    ' Retrieves all attributes from a specified resource in an object.
    Function GetResourceAttributes(ByVal pszObjectID As String,
                                   ByRef Key As WpdStruct.PROPERTYKEY) As PortableDeviceValues
        Dim ppResourceAttributes As New PortableDeviceValues
        Dim ret = _ifc.GetResourceAttributes(pszObjectID, Key, ppResourceAttributes._ifc)
        If ret = S_OK Then
            Return ppResourceAttributes
        ElseIf ret < S_OK Then
            Throw New COMException("GetResourceAttributes faild", ret)
        End If
        Return Nothing
    End Function

    ' Gets an IStream interface with which to read or write the content data in an object on a device.
    Function GetStream(ByVal pszObjectID As String,
                       ByRef Key As WpdStruct.PROPERTYKEY,
                       ByVal dwMode As WpdStruct.STGM,
                       ByRef pdwOptimalBufferSize As UInteger) As Stream
        Dim ppStream As New Stream
        Dim ret = _ifc.GetStream(pszObjectID, Key, dwMode, pdwOptimalBufferSize, ppStream._ifc)
        If ret = S_OK Then
            Return ppStream
        ElseIf ret < S_OK Then
            Throw New COMException("GetStream faild", ret)
        End If
        Return Nothing
    End Function

    ' Deletes one or more resources from an object.
    Function Delete(ByVal pszObjectID As String,
                    ByRef pKeys As PortableDeviceKeyCollection) As Boolean
        Dim ret = _ifc.Delete(pszObjectID, pKeys._ifc)
        If ret = S_OK Then
            Return True
        ElseIf ret < S_OK Then
            Throw New COMException("Delete faild", ret)
        End If
        Return False
    End Function

    ' Cancels a pending operation on this interface.
    Function Cancel() As Boolean
        Dim ret = _ifc.Cancel
        If ret = S_OK Then
            Return True
        ElseIf ret < S_OK Then
            Throw New COMException("Cancel faild", ret)
        End If
        Return False
    End Function
#End Region

#Region "IDisposable Support"
    Private disposedValue As Boolean

    Protected Overridable Sub Dispose(disposing As Boolean)
        If Not disposedValue Then
            If disposing Then
                ' TODO: verwalteten Zustand (verwaltete Objekte) entsorgen.
            End If

            If _ifc IsNot Nothing Then
                Marshal.ReleaseComObject(_ifc)
                _ifc = Nothing
            End If
        End If
        disposedValue = True
    End Sub

    Protected Overrides Sub Finalize()
        Dispose(False)
        MyBase.Finalize()
    End Sub

    Public Sub Dispose() Implements IDisposable.Dispose
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub
#End Region

End Class
