﻿' Autor: Frank Schüler
' Datum: 01.01.2018
' Status: in Arbeit

Option Strict On
Option Explicit On

Imports System.Runtime.InteropServices

Public Class PortableDeviceValues
    Implements IDisposable

    Private Const IID_IPortableDeviceValues As String = "6848F6F2-3155-4F86-B6F5-263EEEAB3143"
    Private Const CLSID_PortableDeviceValues As String = "0C15D503-D017-47CE-9016-7B3F978721CC"
    Private Const S_OK As Integer = 0

    Public _ifc As IPortableDeviceValues

    Sub New()
        _ifc = DirectCast(Activator.CreateInstance(Type.GetTypeFromCLSID(New Guid(CLSID_PortableDeviceValues))), IPortableDeviceValues)
    End Sub

#Region "Interface IPortableDeviceValues"
    ' https://msdn.microsoft.com/en-us/library/windows/desktop/dd319461(v=vs.85).aspx
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IPortableDeviceValues)>
    Public Interface IPortableDeviceValues

        Function GetCount(<Out> ByRef pcelt As UInteger) As Integer

        Function GetAt(<[In]> ByVal index As UInteger,
                       <[In], Out, MarshalAs(UnmanagedType.Struct)> ByVal pkey As WpdStruct.PROPERTYKEY,
                       <[In], Out, MarshalAs(UnmanagedType.Struct)> ByVal pValue As WpdStruct.PROPVARIANT) As Integer

        Function SetValue(<[In], MarshalAs(UnmanagedType.Struct)> ByRef key As WpdStruct.PROPERTYKEY,
                          <[In], MarshalAs(UnmanagedType.Struct)> ByRef Value As WpdStruct.PROPVARIANT) As Integer

        Function GetValue(<[In], MarshalAs(UnmanagedType.Struct)> ByRef key As WpdStruct.PROPERTYKEY,
                          <Out, MarshalAs(UnmanagedType.Struct)> ByRef pValue As WpdStruct.PROPVARIANT) As Integer

        Function SetStringValue(<[In], MarshalAs(UnmanagedType.Struct)> ByRef key As WpdStruct.PROPERTYKEY,
                                <[In], MarshalAs(UnmanagedType.LPWStr)> ByVal Value As String) As Integer

        Function GetStringValue(<[In], MarshalAs(UnmanagedType.Struct)> ByRef key As WpdStruct.PROPERTYKEY,
                                <Out, MarshalAs(UnmanagedType.LPWStr)> ByRef pValue As String) As Integer

        Function SetUnsignedIntegerValue(<[In], MarshalAs(UnmanagedType.Struct)> ByRef key As WpdStruct.PROPERTYKEY,
                                         ByVal Value As UInteger) As Integer

        Function GetUnsignedIntegerValue(<[In], MarshalAs(UnmanagedType.Struct)> ByRef key As WpdStruct.PROPERTYKEY,
                                         ByRef pValue As UInteger) As Integer

        Function SetSignedIntegerValue(<[In], MarshalAs(UnmanagedType.Struct)> ByRef key As WpdStruct.PROPERTYKEY,
                                       ByVal Value As Integer) As Integer

        Function GetSignedIntegerValue(<[In], MarshalAs(UnmanagedType.Struct)> ByRef key As WpdStruct.PROPERTYKEY,
                                       ByRef pValue As Integer) As Integer

        Function SetUnsignedLargeIntegerValue(<[In], MarshalAs(UnmanagedType.Struct)> ByRef key As WpdStruct.PROPERTYKEY,
                                              ByVal Value As ULong) As Integer

        Function GetUnsignedLargeIntegerValue(<[In], MarshalAs(UnmanagedType.Struct)> ByRef key As WpdStruct.PROPERTYKEY,
                                              ByRef pValue As ULong) As Integer

        Function SetSignedLargeIntegerValue(<[In], MarshalAs(UnmanagedType.Struct)> ByRef key As WpdStruct.PROPERTYKEY,
                                            ByVal Value As Long) As Integer

        Function GetSignedLargeIntegerValue(<[In], MarshalAs(UnmanagedType.Struct)> ByRef key As WpdStruct.PROPERTYKEY,
                                            ByRef pValue As Long) As Integer

        Function SetFloatValue(<[In], MarshalAs(UnmanagedType.Struct)> ByRef key As WpdStruct.PROPERTYKEY,
                               ByVal Value As Single) As Integer

        Function GetFloatValue(<[In], MarshalAs(UnmanagedType.Struct)> ByRef key As WpdStruct.PROPERTYKEY,
                               ByRef pValue As Single) As Integer

        Function SetErrorValue(<[In], MarshalAs(UnmanagedType.Struct)> ByRef key As WpdStruct.PROPERTYKEY,
                               <[In], MarshalAs(UnmanagedType.Error)> ByVal Value As Integer) As Integer

        Function GetErrorValue(<[In], MarshalAs(UnmanagedType.Struct)> ByRef key As WpdStruct.PROPERTYKEY,
                               <Out, MarshalAs(UnmanagedType.Error)> ByRef pValue As Integer) As Integer

        Function SetKeyValue(<[In], MarshalAs(UnmanagedType.Struct)> ByRef key As WpdStruct.PROPERTYKEY,
                             <[In], MarshalAs(UnmanagedType.Struct)> ByRef Value As WpdStruct.PROPERTYKEY) As Integer

        Function GetKeyValue(<[In], MarshalAs(UnmanagedType.Struct)> ByRef key As WpdStruct.PROPERTYKEY,
                             <Out, MarshalAs(UnmanagedType.Struct)> ByRef pValue As WpdStruct.PROPERTYKEY) As Integer

        Function SetBoolValue(<[In], MarshalAs(UnmanagedType.Struct)> ByRef key As WpdStruct.PROPERTYKEY,
                              ByVal Value As Integer) As Integer

        Function GetBoolValue(<[In], MarshalAs(UnmanagedType.Struct)> ByRef key As WpdStruct.PROPERTYKEY,
                              ByRef pValue As Integer) As Integer

        Function SetIUnknownValue(<[In], MarshalAs(UnmanagedType.Struct)> ByRef key As WpdStruct.PROPERTYKEY,
                                  <[In], MarshalAs(UnmanagedType.IUnknown)> ByVal Value As IntPtr) As Integer

        Function GetIUnknownValue(<[In], MarshalAs(UnmanagedType.Struct)> ByRef key As WpdStruct.PROPERTYKEY,
                                  <Out, MarshalAs(UnmanagedType.IUnknown)> ByRef pValue As IntPtr) As Integer

        Function SetGuidValue(<[In], MarshalAs(UnmanagedType.Struct)> ByRef key As WpdStruct.PROPERTYKEY,
                              <[In], MarshalAs(UnmanagedType.Struct)> ByVal Value As Guid) As Integer

        Function GetGuidValue(<[In], MarshalAs(UnmanagedType.Struct)> ByRef key As WpdStruct.PROPERTYKEY,
                              <Out, MarshalAs(UnmanagedType.Struct)> ByRef pValue As Guid) As Integer

        ' ToDo
        Function SetBufferValue() As Integer

        ' ToDo
        Function GetBufferValue() As Integer

        ' ToDo
        Function SetIPortableDeviceValuesValue() As Integer

        ' ToDo
        Function GetIPortableDeviceValuesValue() As Integer

        ' ToDo
        Function SetIPortableDevicePropVariantCollectionValue() As Integer

        ' ToDo
        Function GetIPortableDevicePropVariantCollectionValue() As Integer

        ' ToDo
        Function SetIPortableDeviceKeyCollectionValue() As Integer

        ' ToDo
        Function GetIPortableDeviceKeyCollectionValue() As Integer

        ' ToDo
        Function SetIPortableDeviceValuesCollectionValue() As Integer

        ' ToDo
        Function GetIPortableDeviceValuesCollectionValue() As Integer

        Function RemoveValue(<[In], MarshalAs(UnmanagedType.Struct)> ByRef key As WpdStruct.PROPERTYKEY) As Integer

        ' ToDo
        Function CopyValuesFromPropertyStore() As Integer

        ' ToDo
        Function CopyValuesToPropertyStore() As Integer

        Function Clear() As Integer

    End Interface
#End Region

#Region "Interface Helper"
    Public Function GetCount() As UInteger
        Dim pcelt As UInteger
        Dim ret As Integer = _ifc.GetCount(pcelt)
        If ret = S_OK Then
            Return pcelt
        ElseIf ret < S_OK Then
            Throw New COMException("SetStringValue faild", ret)
        End If
        Return 0
    End Function

    Public Function SetStringValue(ByVal Key As WpdStruct.PROPERTYKEY, ByVal Value As String) As Boolean
        Dim ret As Integer = _ifc.SetStringValue(Key, Value)
        If ret = S_OK Then
            Return True
        ElseIf ret < S_OK Then
            Throw New COMException("SetStringValue faild", ret)
        End If
        Return False
    End Function

    Public Function GetStringValue(ByRef Key As WpdStruct.PROPERTYKEY) As String
        Dim pValue As String = String.Empty
        Try
            Dim ret As Integer = _ifc.GetStringValue(Key, pValue)
            If ret = S_OK Then
                Return pValue
            End If
        Catch ex As Exception
            'Throw New COMException("GetStringValue faild", ret)
        End Try
        Return pValue
    End Function

    Function SetUnsignedIntegerValue(ByRef key As WpdStruct.PROPERTYKEY, ByVal Value As UInteger) As Boolean
        Dim ret As Integer = _ifc.SetUnsignedIntegerValue(key, Value)
        If ret = S_OK Then
            Return True
        ElseIf ret < S_OK Then
            Throw New COMException("SetUnsignedIntegerValue faild", ret)
        End If
        Return False
    End Function

    Function GetUnsignedIntegerValue(ByRef key As WpdStruct.PROPERTYKEY) As UInteger
        Dim pValue As UInteger
        Try
            Dim ret As Integer = _ifc.GetUnsignedIntegerValue(key, pValue)
            If ret = S_OK Then
                Return pValue
            End If
        Catch ex As Exception
            'Throw New COMException("GetUnsignedIntegerValue faild", ret)
        End Try
        Return pValue
    End Function

    Function SetSignedIntegerValue(ByRef key As WpdStruct.PROPERTYKEY, ByVal Value As Integer) As Boolean
        Dim ret As Integer = _ifc.SetSignedIntegerValue(key, Value)
        If ret = S_OK Then
            Return True
        ElseIf ret < S_OK Then
            Throw New COMException("SetSignedIntegerValue faild", ret)
        End If
        Return False
    End Function

    Function GetSignedIntegerValue(ByRef key As WpdStruct.PROPERTYKEY) As Integer
        Dim pValue As Integer
        Try
            Dim ret As Integer = _ifc.GetSignedIntegerValue(key, pValue)
            If ret = S_OK Then
                Return pValue
            End If
        Catch ex As Exception
            'Throw New COMException("GetSignedIntegerValue faild", ret)
        End Try
        Return pValue
    End Function


    Function SetBoolValue(ByRef key As WpdStruct.PROPERTYKEY, ByVal Value As Boolean) As Boolean
        Dim ret As Integer = _ifc.SetBoolValue(key, CInt(Value))
        If ret = S_OK Then
            Return True
        ElseIf ret < S_OK Then
            Throw New COMException("SetBoolValue faild", ret)
        End If
        Return False
    End Function

    Function GetBoolValue(ByRef key As WpdStruct.PROPERTYKEY) As Boolean
        Dim pValue As Integer
        Try
            Dim ret As Integer = _ifc.GetBoolValue(key, pValue)
            If ret = S_OK Then
                Return CBool(pValue)
            End If
        Catch ex As Exception
            'Throw New COMException("GetBoolValue faild", ret)
        End Try
        Return False
    End Function

    Public Function RemoveValue(ByVal Key As WpdStruct.PROPERTYKEY) As Boolean
        Dim ret As Integer = _ifc.RemoveValue(Key)
        If ret = S_OK Then
            Return True
        ElseIf ret < S_OK Then
            Throw New COMException("GetStringValue faild", ret)
        End If
        Return False
    End Function

    Public Function Clear() As Boolean
        Dim ret As Integer = _ifc.Clear
        If ret = S_OK Then
            Return True
        ElseIf ret < S_OK Then
            Throw New COMException("Clear faild", ret)
        End If
        Return False
    End Function
#End Region

#Region "IDisposable Support"
    Private disposedValue As Boolean

    Protected Overridable Sub Dispose(disposing As Boolean)
        If Not disposedValue Then
            If disposing Then
                ' TODO: verwalteten Zustand (verwaltete Objekte) entsorgen.
            End If

            If _ifc IsNot Nothing Then
                Marshal.ReleaseComObject(_ifc)
                _ifc = Nothing
            End If
        End If
        disposedValue = True
    End Sub

    Protected Overrides Sub Finalize()
        Dispose(False)
        MyBase.Finalize()
    End Sub

    Public Sub Dispose() Implements IDisposable.Dispose
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub
#End Region

End Class



