﻿' Autor: Frank Schüler
' Datum: 01.01.2018
' Status: in Arbeit

Option Strict On
Option Explicit On

Imports System.Runtime.InteropServices

Public Class SequentialStream
    Implements IDisposable

    Private Const IID_ISequentialStream As String = "0C733A30-2A1C-11CE-ADE5-00AA0044773D"
    Private Const CLSID_SequentialStream As String = ""
    Private Const S_OK As Integer = 0

    Public _ifc As ISequentialStream

#Region "Interface ISequentialStream"
    ' https://msdn.microsoft.com/de-de/library/windows/desktop/aa380010(v=vs.85).aspx
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_ISequentialStream)>
    Public Interface ISequentialStream
        Function Read(<Out, MarshalAs(UnmanagedType.LPArray, SizeParamIndex:=1)> ByVal pv As Byte(),
                      <[In]> ByVal cb As UInteger,
                      <Out> ByRef pcbRead As UInteger) As Integer

        Function Write(<[In], MarshalAs(UnmanagedType.LPArray, SizeParamIndex:=1)> ByVal pv As Byte(),
                       <[In]> ByVal cb As UInteger,
                       <Out> ByRef pcbWritten As UInteger) As Integer

    End Interface
#End Region

#Region "Interface Helper"
    Function Read(ByRef pv As Byte(), ByVal cb As UInteger) As UInteger
        Dim pcbRead As UInteger
        Dim ret = _ifc.Read(pv, cb, pcbRead)
        If ret = S_OK Then
            Return pcbRead
        ElseIf ret < S_OK Then
            Throw New COMException("Read faild", ret)
        End If
        Return 0
    End Function

    Function Write(ByRef pv As Byte(), ByVal cb As UInteger) As UInteger
        Dim pcbWritten As UInteger
        Dim ret = _ifc.Read(pv, cb, pcbWritten)
        If ret = S_OK Then
            Return pcbWritten
        ElseIf ret < S_OK Then
            Throw New COMException("Write faild", ret)
        End If
        Return 0
    End Function
#End Region

#Region "IDisposable Support"
    Private disposedValue As Boolean

    Protected Overridable Sub Dispose(disposing As Boolean)
        If Not disposedValue Then
            If disposing Then
                ' TODO: verwalteten Zustand (verwaltete Objekte) entsorgen.
            End If

            If _ifc IsNot Nothing Then
                Marshal.ReleaseComObject(_ifc)
                _ifc = Nothing
            End If
        End If
        disposedValue = True
    End Sub

    Protected Overrides Sub Finalize()
        Dispose(False)
        MyBase.Finalize()
    End Sub

    Public Sub Dispose() Implements IDisposable.Dispose
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub
#End Region

End Class
