﻿' Autor: Frank Schüler
' Datum: 01.01.2018
' Status: in Arbeit

Option Strict On
Option Explicit On

Imports System.Runtime.InteropServices

Public Class Stream
    Implements IDisposable

    Private Const IID_IStream As String = "0000000C-0000-0000-C000-000000000046"
    Private Const CLSID_Stream As String = ""
    Private Const S_OK As Integer = 0

    Public _ifc As IStream

#Region "Interface IStream"
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IStream)>
    Public Interface IStream
        Inherits SequentialStream.ISequentialStream

        Shadows Function Read(<Out, MarshalAs(UnmanagedType.LPArray, SizeParamIndex:=1)> ByVal pv As Byte(),
                              <[In]> ByVal cb As UInteger,
                              <Out> ByRef pcbRead As UInteger) As Integer

        Shadows Function Write(<[In], MarshalAs(UnmanagedType.LPArray, SizeParamIndex:=1)> ByVal pv As Byte(),
                               <[In]> ByVal cb As UInteger,
                               <Out> ByRef pcbWritten As UInteger) As Integer

        Function Seek(<[In]> ByVal dlibMove As Long,
                      <[In]> ByVal dwOrigin As Integer,
                      <Out> ByRef plibNewPosition As Long) As Integer

        Function SetSize(<[In]> ByVal libNewSize As Long) As Integer

        Function CopyTo(<[In], MarshalAs(UnmanagedType.Interface)> ByVal pstm As Stream.IStream,
                        <[In]> ByVal cb As Long,
                        <Out> ByRef pcbRead As Long,
                        <Out> ByRef pcbWritten As Long) As Integer

        Function Commit(<[In]> ByVal grfCommitFlags As Integer) As Integer

        Function Revert() As Integer

        Function LockRegion(<[In]> ByVal libOffset As Long,
                            <[In]> ByVal cb As Long,
                            <[In]> ByVal dwLockType As Integer) As Integer

        Function UnlockRegion(<[In]> ByVal libOffset As Long,
                              <[In]> ByVal cb As Long,
                              <[In]> ByVal dwLockType As Integer) As Integer

        Function Stat(<Out, MarshalAs(UnmanagedType.Struct)> ByRef pstatstg As WpdStruct.STATSTG,
                      <[In]> ByVal grfStatFlag As Integer) As Integer

        Function Clone(<Out, MarshalAs(UnmanagedType.Interface)> ByRef ppstm As Stream.IStream) As Integer

    End Interface
#End Region

#Region "Interface Helper"
    Function Read(ByRef pv As Byte(), ByVal cb As UInteger) As UInteger
        Dim pcbRead As UInteger
        Dim ret = _ifc.Read(pv, cb, pcbRead)
        If ret = S_OK Then
            Return pcbRead
        ElseIf ret < S_OK Then
            Throw New COMException("Read faild", ret)
        End If
        Return 0
    End Function

    Function Write(ByRef pv As Byte(), ByVal cb As UInteger) As UInteger
        Dim pcbWritten As UInteger
        Dim ret = _ifc.Read(pv, cb, pcbWritten)
        If ret = S_OK Then
            Return pcbWritten
        ElseIf ret < S_OK Then
            Throw New COMException("Write faild", ret)
        End If
        Return 0
    End Function

    Function Seek(ByVal dlibMove As Long, ByVal dwOrigin As WpdStruct.STREAM_SEEK) As Long
        Dim plibNewPosition As Long
        Dim ret = _ifc.Seek(dlibMove, dwOrigin, plibNewPosition)
        If ret = S_OK Then
            Return plibNewPosition
        ElseIf ret < S_OK Then
            Throw New COMException("Seek faild", ret)
        End If
        Return 0
    End Function

    Function SetSize(ByVal libNewSize As Long) As Boolean
        Dim ret = _ifc.SetSize(libNewSize)
        If ret = S_OK Then
            Return True
        ElseIf ret < S_OK Then
            Throw New COMException("SetSize faild", ret)
        End If
        Return False
    End Function

    Function CopyTo(ByVal pstm As Stream, ByVal cb As Long, ByRef pcbRead As Long) As Long
        Dim pcbWritten As Long
        Dim ret = _ifc.CopyTo(pstm._ifc, cb, pcbRead, pcbWritten)
        If ret = S_OK Then
            Return pcbWritten
        ElseIf ret < S_OK Then
            Throw New COMException("CopyTo faild", ret)
        End If
        Return 0
    End Function

    Function Commit(Optional ByVal grfCommitFlags As WpdStruct.STGC = WpdStruct.STGC.STGC_DEFAULT) As Boolean
        Dim ret = _ifc.Commit(grfCommitFlags)
        If ret = S_OK Then
            Return True
        ElseIf ret < S_OK Then
            Throw New COMException("Commit faild", ret)
        End If
        Return False
    End Function

    Function Revert() As Boolean
        Dim ret = _ifc.Revert
        If ret = S_OK Then
            Return True
        ElseIf ret < S_OK Then
            Throw New COMException("Revert faild", ret)
        End If
        Return False
    End Function

    Function LockRegion() As Integer

    End Function

    Function UnlockRegion() As Integer

    End Function

    Function Stat() As Integer

    End Function

    Function Clone() As Integer

    End Function
#End Region

#Region "IDisposable Support"
    Private disposedValue As Boolean

    Protected Overridable Sub Dispose(disposing As Boolean)
        If Not disposedValue Then
            If disposing Then
                ' TODO: verwalteten Zustand (verwaltete Objekte) entsorgen.
            End If

            If _ifc IsNot Nothing Then
                Marshal.ReleaseComObject(_ifc)
                _ifc = Nothing
            End If
        End If
        disposedValue = True
    End Sub

    Protected Overrides Sub Finalize()
        Dispose(False)
        MyBase.Finalize()
    End Sub

    Public Sub Dispose() Implements IDisposable.Dispose
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub
#End Region

End Class
